// Commands.cpp
//

#include "stdafx.h"

#define MAX_STR 256
char str[MAX_STR];
BOOL details = 2;

#define DISPLAY_HELP(c) {	\
	char *h[1];				\
	h[0] = c;				\
	CmdHelp(1,h);			\
}

int CmdUnknownCommand(int argc, char* argv[])
{
	if (argc == 0)
		printf("Unknown Command!\n");
	else
		printf("Unknown Command: %s", argv[0]);

	return 0;
}

int CmdPrompt(int argc, char *argv[])
{
	if (argc == 0)
	{
		printf("Please provide a new prompt\n");
		DISPLAY_HELP("prompt");
		return 0;
	}

	memset(prompt, 0, MAX_PROMPT);
	strncpy_s(prompt, MAX_PROMPT, argv[0], MAX_PROMPT-1);

	return 0;
}

int CmdHelp(int argc, char *argv[])
{
	int		ndx;
	char	cmd[100];
	char	args[400];
	int		i;
	size_t	cmdLen;
	char	*token,
            *next_token;
	char	*command = NULL;
	BOOL	printed = FALSE;

	for (i = 0; i < argc; i++)
	{
		if (_strnicmp(argv[i], "-d", 2) == 0)
		{
			details = atoi(&(argv[i][2]));
		}
		if (argv[i][0] != '-')
		{
			command = argv[i];
		}
	}

	for (ndx = 1; ndx < NumCmds; ndx++)
	{
		if ((command == NULL) || (_strcmpi(command, Commands[ndx].command) == 0))
		{
			printed = TRUE;

			if (Commands[ndx].cmdArgs == NULL)
			{
				if (details == 0)
					printf("\n%s", Commands[ndx].command);
				else
					printf("\n%-17s %s", Commands[ndx].command, Commands[ndx].comments);
			}
			else
			{
				strcpy_s(cmd, 100, Commands[ndx].command);

				for (i = 0; i < Commands[ndx].cmdArgs->numArgs; i++)
				{
					strcat_s(cmd, 100, " " );
					strcat_s(cmd, 100, Commands[ndx].cmdArgs->args[i].arg);
				}

				cmdLen = strlen(cmd);
				
				if ((details == 0) || (cmdLen <= 17))
				{
					if (details == 0)
						printf("\n%s", cmd);
					else
						printf("\n%-17s %s", cmd, Commands[ndx].comments);
				}
				else
				{
					printf("\n\n%s", cmd);
					printf("\n%-17s %s", "", Commands[ndx].comments);
				}

				if (details >= 2)
				{
					for (i = 0; i < Commands[ndx].cmdArgs->numArgs; i++)
					{
						strcpy_s(args, 400, Commands[ndx].cmdArgs->args[i].usage);
						token = strtok_s(args, "@", &next_token);
						while( token != NULL )
						{
							// While there are tokens
							printf("\n                    %s", token);
							// Get next token: 
							token = strtok_s(NULL, "@", &next_token); // C4996
						}
					}
				}
			}
		}
	}

	if (!printed && command)
	{
		printf("Unknown command %s\n", command);
	}
	else
		printf("\n");

	return 0;
}

int CmdQuit(int argc, char *argv[])
{
	printf("\nUser terminated session.\n");
	exit(0);
	return 0;
}

int CmdFindDevices(int argc, char* argv[])
{
	PCprDeviceInfo	pDevInfo;
	int numDevices;
	int ndx;
	int i;
	int nResult;
	BOOL printNames = FALSE;
	int timeout = 1000;

	for (i = 0; i < argc; i++)
	{
		if (_strcmpi(argv[i], "-n") == 0)
		{
			printNames = TRUE;
		}
		else if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 1000;
		}
	}

	printf("Finding Devices - Names:%s timeout:%d\n", (printNames) ? "Yes" : "No", timeout);

	nResult = CprFindDevices(&pDevInfo, &numDevices, timeout);

	if (nResult == CPR_OK)
	{
        printf("%d devices found!\n", numDevices);

		for (ndx = 0; ndx < numDevices; ndx++)
		{
			if (pDevInfo[ndx].id[0] == '\0')
			{
				pDevInfo[ndx].id[0] = '?';
				pDevInfo[ndx].id[1] = '?';
				pDevInfo[ndx].id[2] = '\0';
			}

			printf("  %-4d  %-15s  %-3s  %02X-%02X-%02X-%02X-%02X-%02X\n",
				ndx+1,
				pDevInfo[ndx].ipAddr,     pDevInfo[ndx].id,
				pDevInfo[ndx].macAddr[0], pDevInfo[ndx].macAddr[1], 
				pDevInfo[ndx].macAddr[2], pDevInfo[ndx].macAddr[3], 
				pDevInfo[ndx].macAddr[4], pDevInfo[ndx].macAddr[5]);

			if (printNames)
			{
				printf("         Device Name     : %s\n", pDevInfo[ndx].devName);
				printf("         Tcp Port 1 Name : %s\n", pDevInfo[ndx].port1Name);
				printf("         Tcp Port 2 Name : %s\n\n", pDevInfo[ndx].port2Name);
			}
		}

		////////////////////////////////////////////////////////////
		// FreeBuffer
		////////////////////////////////////////////////////////////
		nResult = CprFreeBuffer((PVOID)pDevInfo);

		if (nResult != CPR_OK)
		{
	        PrintError("Error calling FreeBuffer.", nResult);
		}
	}
	else
	{
        PrintError("Error calling FindDevices.", nResult);
	}

	return 0;
}

int CmdQuery(int argc, char* argv[])
{
	CprDeviceInfo	devInfo;
	DWORD devInfoSize = sizeof(CprDeviceInfo);
	int nResult = 0;
	int timeout = 500;
	int i;

	if (argc == 0)
	{
		printf("Please provide an IP Address\n");
		DISPLAY_HELP("query");
		return 0;
	}

	for (i = 1; i < argc; i++)
	{
		if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 500;
		}
	}

	printf("Querying Device - IP:%s timeout:%d\n", argv[0], timeout);

	nResult = CprQueryDevice(
				argv[0],
				&devInfo,
				&devInfoSize,
				timeout);

	if (nResult == CPR_OK)
	{
		if (devInfo.id[0] == '\0')
		{
			devInfo.id[0] = '?';
			devInfo.id[1] = '?';
			devInfo.id[2] = '\0';
		}

		printf("        %-15s  %-3s  %02X-%02X-%02X-%02X-%02X-%02X\n",
			argv[0], devInfo.id,
			devInfo.macAddr[0], devInfo.macAddr[1], 
			devInfo.macAddr[2], devInfo.macAddr[3], 
			devInfo.macAddr[4], devInfo.macAddr[5]);
	}
	else if (nResult == CPR_DEVICE_NOT_FOUND)
	{
		printf("        %-15s  Device Not Found\n", argv[0]);
	}
	else
	{
		printf("        %-15s  Error calling QueryDevice\n", argv[0]);
	}

	return 0;
}

int CmdGetLogState(int argc, char* argv[])
{
	int nResult;
	BOOL loggingState;

	nResult = CprGetCprIfLoggingState(&loggingState);

	if (nResult == CPR_OK)
	{
		printf("Logging is %s\n", (loggingState) ? "On" : "Off");
	}
	else
	{
		PrintError("Error retreiving CPR Logging State (CprGetCprIfLoggingState)\n", nResult);
	}

	return 0;
}

int CmdSetLogState(int argc, char* argv[])
{
	int nResult;
	BOOL loggingState;

	if (argc == 0)
	{
		printf("Please provide a new logging state\n");
		DISPLAY_HELP("setlogstate");
		return 0;
	}

	loggingState = (*argv[0] == '0') ? FALSE : TRUE;

	nResult = CprSetCprIfLoggingState(loggingState);

	if (nResult == CPR_OK)
	{
		printf("Call to CprSetCprIfLoggingState was successful\n");
	}
	else
	{
		PrintError("Error setting CPR Logging State (CprGetCprIfLoggingState)\n", nResult);
	}

	nResult = CprGetCprIfLoggingState(&loggingState);

	if (nResult == CPR_OK)
	{
		printf("Logging is %s\n", (loggingState) ? "On" : "Off");
	}
	else
	{
		PrintError("Error retreiving CPR Logging State (CprGetCprIfLoggingState)\n", nResult);
	}

	return 0;
}

int CmdIsReachable(int argc, char* argv[])
{
	int nResult;
	int timeout = 500;
	int i;

	if (argc == 0)
	{
		printf("Please provide an IP Address\n");
		DISPLAY_HELP("reach");
		return 0;
	}

	for (i = 1; i < argc; i++)
	{
		if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 500;
		}
	}

	printf("Contacting Device - IP:%s timeout:%d\n", argv[0], timeout);

	nResult = CprIsDeviceReachable(argv[0], timeout);

	if (nResult == CPR_OK)
	{
		printf("  Device is reachable\n");
	}
	else if (nResult == CPR_DEVICE_NOT_FOUND)
	{
		printf("  Device is not reachable\n");
	}
	else
	{
		printf("  Error calling IsDeviceReachable\n");
	}

	return 0;
}

int CmdIsReachableEx(int argc, char* argv[])
{
	int nResult;
	int timeout = 500;
	int i;

	if (argc == 0)
	{
		printf("Please provide an IP Address\n");
		DISPLAY_HELP("reach");
		return 0;
	}

	for (i = 1; i < argc; i++)
	{
		if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 500;
		}
	}

	printf("Contacting Device - IP:%s timeout:%d\n", argv[0], timeout);

	nResult = CprIsDeviceReachableEx(argv[0], TRUE, timeout);

	if (nResult == CPR_OK)
	{
		printf("  Device is reachable\n");
	}
	else if (nResult == CPR_DEVICE_NOT_FOUND)
	{
		printf("  Device is not reachable\n");
	}
	else
	{
		printf("  Error calling IsDeviceReachable\n");
	}

	return 0;
}

int CmdGetVersions(int argc, char* argv[])
{
	int		nResult;
    char	versionString[50];
    DWORD	versionStringLen = 50;

    nResult = CprGetCprIfVersion(((char*)(&versionString)),&versionStringLen);
    if (nResult == CPR_OK)
	{
		printf("CPR interface version : %s\n", versionString);
    }
    else
    {
		PrintError("Error retreiving CPR interface version (GetCprDrvrVersion)\n", nResult);
    }

    versionStringLen = 50;
    nResult = CprGetCprDrvrVersion(((char*)(&versionString)),&versionStringLen);
    if (nResult == CPR_OK)
	{
		printf("CPR driver version    : %s\n", versionString);
    }
    else
    {
		PrintError("Error retreiving CPR driver version (GetCprDrvrVersion)\n",nResult);
    }

	return 0;
}

int CmdGetDefault(int argc, char* argv[])
{
	int			nResult;
    PortInfo	portInfo;

	printf("Default Port Info:\n");
    nResult = CprGetDefaultPortInfo(&portInfo);
    if (nResult == CPR_OK)
    {
        LogPortInfo(&portInfo);
    }
    else
    {
        PrintError("Error retreiving default port info (GetDefaultPortInfo)", nResult);
    }

	return 0;
}

void LogPortInfo(PortInfo* p)
{
	char *pStr;

    printf("portNumber        : %d\n", p->portNumber);
    printf("portName          : %ws\n", p->portName);
    printf("AES               : %s\n", TO_BOOL(p->AES));

    if (p->AES != 1) 
    {
        printf(
			"AESKey1           : %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
			p->AESKey[0], p->AESKey[1], p->AESKey[2], p->AESKey[3],
			p->AESKey[4], p->AESKey[5], p->AESKey[6], p->AESKey[7],
			p->AESKey[8], p->AESKey[9], p->AESKey[10], p->AESKey[11],
			p->AESKey[12], p->AESKey[13], p->AESKey[14], p->AESKey[15]);
        printf(
			"AESKey2           : %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
			p->AESKey[16], p->AESKey[17], p->AESKey[18], p->AESKey[19],
			p->AESKey[20], p->AESKey[21], p->AESKey[22], p->AESKey[23],
			p->AESKey[24], p->AESKey[25], p->AESKey[26], p->AESKey[27],
			p->AESKey[28], p->AESKey[29], p->AESKey[30], p->AESKey[31]);
    }

    printf("AESKeyLength      : %d\n", p->AESKeyLength);
    printf("TimeoutReconnect  : %s\n", TO_BOOL(p->AutoReconErr));
    printf("ServerReconnect   : %s\n", TO_BOOL(p->AutoReconPeer));
    printf("ReconnectLimit    : %d\n", p->ReconLimit);
    printf("ServerReconnect   : %s\n", TO_BOOL(p->AutoReconPeer));
    printf("ConnectionTimeout : %d\n", p->CnTmo);
    printf("NoNetClose        : %s\n", TO_BOOL(p->NoDiscon));
    printf("UseRFC2217        : %s\n", TO_BOOL(p->UseRFC2217));
	printf("ListenMode        : %s\n", p->ListenMode ? ((p->ListenMode == 1) ? "Normal" : "Auto") : "False");
    printf("TCPKeepAlive      : %s\n", TO_BOOL(p->KeepAliveOnOff));
    printf("KeepAliveTime     : %d\n", p->KeepAliveTime);
    printf("KeepAliveInterval : %d\n", p->KeepAliveInterval);

	pStr = NULL;

    switch (p->DtrConnection)
    {
		case DTR_TO_DCD_DSR_ACTIVE:
			pStr =  "DtrConnection     : DTR_TO_DCD_DSR_ACTIVE\n";
			break;
		case DTR_TO_DCD_DSR_INACTIVE:
			pStr =  "DtrConnection     : DTR_TO_DCD_DSR_INACTIVE\n";
			break;
		case DTR_TO_DSR_DCD_ACTIVE:
			pStr =  "DtrConnection     : DTR_TO_DSR_DCD_ACTIVE\n";
			break;
		case DTR_TO_DSR_DCD_INACTIVE:
			pStr =  "DtrConnection     : DTR_TO_DSR_DCD_INACTIVE\n";
			break;
		case DTR_TO_DSR_AND_DCD:
			pStr =  "DtrConnection     : DTR_TO_DSR_AND_DCD\n";
			break;
		case DTR_TO_NONE:
			pStr =  "DtrConnection     : DTR_TO_NONE\n";
			break;
		default:
			sprintf_s(str, MAX_STR, "DtrConnection     : %d(Unknown value)\n",p->DtrConnection);
			break;
    }

	if (pStr != NULL)
	    printf(pStr);
	else
	    printf(str);

    printf("BufferWrites      : %s\n", TO_BOOL(p->BufferWrites));
    printf("TransmitEmpty     : %s\n",
		(p->TransmitEmpty==0)
			? "When CPR is finished transmitting"
			: "When the Device Server is finished transmitting");
    printf("NumServices       : %d\n", p->NumServices);

	if (p->ListenMode != 0)
	{
	    printf("Listen Mode Port  : %d\n", p->Services[0].Port);
	}
    else if (p->NumServices > 0)
    {
        for (ULONG i = 0; i<p->NumServices; i++)
        {
            LogCprService(i+1, &(p->Services[i]));
        }
    }
}

void LogCprServices(PortInfo *pPortInfo)
{
	DWORD i;

	for (i = 0; i < pPortInfo->NumServices; i++)
	{
		LogCprService(i+1, &(pPortInfo->Services[i]));
	}
}

void LogCprService(int port, CPR_SERVICE* p)
{
    printf("Service %-3d       : %ws:%d\n", port, p->Address, p->Port ); 
}

int CmdNumPorts(int argc, char* argv[])
{
	int numPorts = 0;
    int nResult = CprGetNumPorts(&numPorts);
    if (nResult == CPR_OK)
    {
		if (numPorts == 1)
			printf("There is 1 CPR port on this system.\n");        
		else
			printf("There are %d CPR ports on this system.\n", numPorts);        
    }
	else if ((nResult == CPR_PORT_KEY_NOT_FOUND) ||
			 (nResult == CPR_PORT_NOT_FOUND))
	{	
			printf("There are no CPR ports on this system.\n");        
	}
    else
    {
        PrintError("Error calling GetNumPorts", nResult);
    }

	return 0;
}

int CmdPortDevice(int argc, char* argv[])
{
	int				nResult;
    CprPortDevice	cprPortDevice;
	int				portNum;

	if (argc == 0)
	{
		printf("Please provide a port number (ports 0 and >%d are invalid).\n", MAX_PORTS);
		DISPLAY_HELP("devinfo");
		return 0;
	}

	portNum = atoi(argv[0]);

	if ((portNum < 1) || (portNum > MAX_PORTS))
	{
		printf("port number must be 1 or greater but not greater than %d.\n", MAX_PORTS);
		return 0;
	}

	printf("Retrieving Device Info for Port %d\n", portNum);

    nResult = CprGetPortDevice(portNum, &cprPortDevice);
    if (nResult == CPR_OK)
    {
		if (cprPortDevice.id[0] == '\0')
		{
			cprPortDevice.id[0] = '?';
			cprPortDevice.id[1] = '?';
			cprPortDevice.id[2] = '?';
		}

        printf("  %-3s  %02X-%02X-%02X-%02X-%02X-%02X\n",
			cprPortDevice.id,
            cprPortDevice.macAddr[0], cprPortDevice.macAddr[1], 
            cprPortDevice.macAddr[2], cprPortDevice.macAddr[3], 
            cprPortDevice.macAddr[4], cprPortDevice.macAddr[5]);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling GetPortDevice", nResult);
    }

	return 0;
}

int CmdPortInfo(int argc, char* argv[])
{
	int			nResult;
    PortInfo	portInfo;
	int			portNum;

	if (argc == 0)
	{
		printf("Please provide a port number (ports 0 and >%d are invalid).\n", MAX_PORTS);
		DISPLAY_HELP("portinfo");
		return 0;
	}

	portNum = atoi(argv[0]);

	if ((portNum < 1) || (portNum > MAX_PORTS))
	{
		printf("port number must be 1 or greater but not greater than %d.\n", MAX_PORTS);
		return 0;
	}

	printf("Retrieving Port Info for Port %d\n", portNum);

    memset((void*)&portInfo,0,sizeof(portInfo));
    nResult = CprGetPortInfo(&portInfo, portNum);
    if (nResult == CPR_OK)
    {
        LogPortInfo(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling GetPortInfo", nResult);
    }

	return 0;
}

int CmdPortMap(int argc, char* argv[])
{
    int		nResult;
    PortMap	portMap;
	DWORD	i;
	
	printf("\nRetrieving CPR Port Map\n");

	nResult = CprGetPortMap(&portMap);
    if (nResult == CPR_OK)
    {
		if (portMap.numPorts == 1)
			printf("There is 1 CPR port in the CPR Port Map:\n\n");
		else
			printf("There are %d CPR ports in the CPR Port Map:\n\n", portMap.numPorts);
		
        for (i = 0; i < portMap.numPorts; i++)
        {
			printf("PortMap[%d]: %d, %ws\n", i, portMap.Map[i].portNumber, portMap.Map[i].portName);
        }
    }
    else
    {
        PrintError("Error calling GetPortMap", nResult);
    }

	return 0;
}

int CmdPortStatus(int argc, char* argv[])
{
	int			nResult;
    PortStatus	portStatus;
	int			portNum;

	if (argc == 0)
	{
		printf("Please provide a port number (ports 0 and >%d are invalid).\n", MAX_PORTS);
		DISPLAY_HELP("portstatus");
		return 0;
	}

	portNum = atoi(argv[0]);

	if ((portNum < 1) || (portNum > MAX_PORTS))
	{
		printf("port number must be 1 or greater but not greater than %d.\n", MAX_PORTS);
		return 0;
	}

	printf("Retrieving Port Status for Port %d\n", portNum);

    memset((void*)&portStatus,0,sizeof(portStatus));
    portStatus.portNumber = portNum;
    nResult = CprGetPortStatus(&portStatus);
    if (nResult == CPR_OK)
    {
        printf("  portNumber : %d\n", portStatus.portNumber);

        switch (portStatus.status)
        {
			case CPR_COM_STATUS_CLOSED:
				printf("  status     : %d (CLOSED)\n", portStatus.status);
				break;
			case CPR_COM_STATUS_OPEN:
				printf("  status     : %d (OPEN)\n", portStatus.status);
				break;
			case CPR_COM_STATUS_ERROR:
				printf("  status     : %d (ERROR)\n", portStatus.status);
				break;
			case CPR_COM_STATUS_OPENING:
				printf("  status     : %d (OPENING)\n", portStatus.status);
				break;
			case CPR_COM_STATUS_CLOSING:
				printf("  status     : %d (CLOSING)\n", portStatus.status);
				break;
			default:
				printf("  status     : %d (!UNKNOWN STATUS!)\n", portStatus.status);
				break;
        }

        switch (portStatus.networkStatus)
        {
			case CPR_NETWORK_STATUS_DISCONNECTED:
				printf("  net status : %d (DISCONNECTE)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_CONNECTING:
				printf("  net status : %d (CONNECTING)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_RECONNECTING:
				printf("  net status : %d (RECONNECTING)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_CONNECTED:
				printf("  net status : %d (CONNECTED)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_LISTENING:
				printf("  net status : %d (LISTENING)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_NEGOTIATING:
				printf("  net status : %d (NEGOTIATING)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_VERIFYING:
				printf("  net status : %d (VERIFYING)\n", portStatus.networkStatus);
				break;
			case CPR_NETWORK_STATUS_ERROR:
				printf("  net status : %d (ERROR)\n", portStatus.networkStatus);
				break;
			default:
				printf("  net status : %d (!UNKNOWN STATUS!)\n", portStatus.networkStatus);
				break;
        }


        printf("  IP Address : %ws\n", portStatus.Address);
        printf("  TCP Port   : %d\n", portStatus.Port);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling GetPortStatus", nResult);
    }

	return 0;
}

int CmdPortUsage(int argc, char* argv[])
{
	int		nResult;
    BYTE	buffer[MAX_PORTS];
    DWORD	bufferSize = MAX_PORTS;
    DWORD	maxPortsReported = 0;
	DWORD	i;
    PortMap	portMap;

	char GetStatChar(DWORD portNumber, BOOL used, PortMap *pPortMap);
	
	printf("\nRetrieving Port Map...\n");
	nResult = CprGetPortMap(&portMap);

	if (nResult != CPR_OK)
	{
        PrintError("Error retrieving Port Map", nResult);
		return 0;
	}

	printf("Retrieving Port Usage...\n");

    nResult = CprGetPortUsage((PBYTE)&buffer, bufferSize, &maxPortsReported);
    if (nResult == CPR_OK)
    {
		//
		// 0 = COM1, 1 = COM2, 2 = COM3, etc.... 255 = COM256.
		//
		printf("' ' : Port is available\n");
		printf("'X' : Port is used, but is not a CPR port\n");;
		printf("'C' : Port is a CPR port\n\n");

        for (i=0; i<MAX_PORTS; i+=13)
        {
			printf("%3d-%c ", i+1, GetStatChar(i+1, buffer[i], &portMap));
			if (i+1 < MAX_PORTS)
				printf("%3d-%c ", i+2, GetStatChar(i+2, buffer[i+1], &portMap));
			if (i+2 < MAX_PORTS)
				printf("%3d-%c ", i+3, GetStatChar(i+3, buffer[i+2], &portMap));
			if (i+3 < MAX_PORTS)
				printf("%3d-%c ", i+4, GetStatChar(i+4, buffer[i+3], &portMap));
			if (i+4 < MAX_PORTS)
				printf("%3d-%c ", i+5, GetStatChar(i+5, buffer[i+4], &portMap));
			if (i+5 < MAX_PORTS)
				printf("%3d-%c ", i+6, GetStatChar(i+6, buffer[i+5], &portMap));
			if (i+6 < MAX_PORTS)
				printf("%3d-%c ", i+7, GetStatChar(i+7, buffer[i+6], &portMap));
			if (i+7 < MAX_PORTS)
				printf("%3d-%c ", i+8, GetStatChar(i+8, buffer[i+7], &portMap));
			if (i+8 < MAX_PORTS)
				printf("%3d-%c ", i+9, GetStatChar(i+9, buffer[i+8], &portMap));
			if (i+9 < MAX_PORTS)
				printf("%3d-%c ", i+10, GetStatChar(i+10, buffer[i+9], &portMap));
			if (i+10 < MAX_PORTS)
				printf("%3d-%c ", i+11, GetStatChar(i+11, buffer[i+10], &portMap));
			if (i+11 < MAX_PORTS)
				printf("%3d-%c ", i+12, GetStatChar(i+12, buffer[i+11], &portMap));
			if (i+12 < MAX_PORTS)
				printf("%3d-%c\n", i+13, GetStatChar(i+13, buffer[i+12], &portMap));
        }
    }
    else
    {
        PrintError("Error retrieving Port Usage", nResult);
    }

	printf("\n");

	return 0;
}

char GetStatChar(DWORD portNumber, BOOL used, PortMap *pPortMap)
{
	DWORD	i;
	char	ch = (used) ? 'X' : ' ';	// blank is not used, 'X' means used but not CPR port

	for (i = 0; i < pPortMap->numPorts; i++)
	{
		if (pPortMap->Map[i].portNumber == portNumber)
		{
			// override used or not with "this is a CPR port'
			ch = 'C';
			break;
		}
	}

	return ch;
}

int CmdAddPort(int argc, char* argv[])
{
	int			nResult;
	int			portNum;
	PortInfo	portInfo;

	if (argc == 0)
	{
		printf("Please provide a port number (ports 0 and >%d are invalid).\n", MAX_PORTS);
		DISPLAY_HELP("addport");
		return 0;
	}

	portNum = atoi(argv[0]);

	if ((portNum < 1) || (portNum > MAX_PORTS))
	{
		printf("port number must be 1 or greater but not greater than %d.\n", MAX_PORTS);
		return 0;
	}

	printf("Adding Port %d...\n", portNum);

    nResult = CprAddPort(portNum, FALSE);
    if (nResult == CPR_OK)
    {
        printf("  Port added successfully\n");
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
		return 0;
	}
    else
    {
        PrintError("Error calling CprAddPort", nResult);
		return 0;
    }

    nResult = CprGetDefaultPortInfo(&portInfo);
    if (nResult != CPR_OK)
    {
        PrintError("Error calling GetDefaultPortInfo", nResult);
		return 0;
    }

	portInfo.portNumber = portNum;
	swprintf(portInfo.portName, 8, L"COM%d", portNum);

    nResult = CprSetPortInfo(&portInfo);
    if (nResult == CPR_OK)
    {
		printf("\nPort Settings: \n");
        LogPortInfo(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling SetPortInfo", nResult);
    }

	return 0;
}

int CmdRemovePort(int argc, char* argv[])
{
	int			nResult;
	int			portNum;

	if (argc == 0)
	{
		printf("Please provide a port number (ports 0 and >%d are invalid).\n", MAX_PORTS);
		DISPLAY_HELP("remport");
		return 0;
	}

	portNum = atoi(argv[0]);

	if ((portNum < 1) || (portNum > MAX_PORTS))
	{
		printf("port number must be 1 or greater but not greater than %d.\n", MAX_PORTS);
		return 0;
	}

	printf("Removing Port %d...\n", portNum);

    nResult = CprRemovePort(portNum);
    if (nResult == CPR_OK)
    {
        printf("  Port removed successfully\n");
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling RemovePort", nResult);
    }

	return 0;
}

int CmdSetDeviceNames(int argc, char* argv[])
{
	int		nResult;
	char*	devName = NULL;
	char*	port1Name = NULL;
	char*	port2Name = NULL;
	int		timeout = 1000;
	int		i;

	if (argc < 2)
	{
		printf("Please provide an IP Address and at least one name.\n");
		DISPLAY_HELP("setnames");
		return 0;
	}

	// Parse args
	for (i = 1; i < argc; i++)
	{
		if (_strnicmp("-d", argv[i], 2) == 0)
		{
			devName = &argv[i][2];
		}
		else if (_strnicmp("-1", argv[i], 2) == 0)
		{
			port1Name = &argv[i][2];
		}
		else if (_strnicmp("-2", argv[i], 2) == 0)
		{
			port2Name = &argv[i][2];
		}
		else if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 1000;
		}
	}

	if (devName)   Trim(devName, '\"');
	if (port1Name) Trim(port1Name, '\"');
	if (port2Name) Trim(port2Name, '\"');

	printf("Setting names for device %s timeout:%d as:\n", argv[0], timeout);
	printf(" Device Name     : %s\n", (devName) ? devName : "(Not setting device name)");
	printf(" Tcp Port 1 Name : %s\n", (port1Name) ? port1Name : "(Not setting port 1 name)");
	printf(" Tcp Port 2 Name : %s\n", (port2Name) ? port2Name : "(Not setting port 2 name)");

	nResult = CprSetDeviceAndPortNames(argv[0], devName, port1Name, port2Name, timeout);
	if (nResult == CPR_OK)
	{
		printf("  Names changed successfully\n");
	}
	else
	{
		PrintError("Error calling SetDeviceAndPortNames", nResult);
	}

	return 0;
}

void Trim(char *str, char tch)
{
	size_t i;
	size_t len = strlen(str);

	for (i = 0; i < len; i++)
	{
		if (str[i] != tch)
			break;
	}

	if (i != 0)
	{
		len -= i;
		memmove(str, &str[i], len);
		str[len] = '\0';
	}

	for (i = len-1; i > 0; i--)
	{
		if (str[i] != tch)
			break;
	}

	str[i+1] = '\0';
}

int CmdGetDeviceNames(int argc, char* argv[])
{
	int		nResult;
	char	devName[INFO_NAME_LEN];
	char	port1Name[INFO_NAME_LEN];
	char	port2Name[INFO_NAME_LEN];
	DWORD	nameLen = INFO_NAME_LEN;
	int		timeout = 1000;
	int		i;

	if (argc == 0)
	{
		printf("Please provide an IP Address.\n");
		DISPLAY_HELP("getnames");
		return 0;
	}

	// Parse args
	for (i = 1; i < argc; i++)
	{
		if (_strnicmp("-t", argv[i], 2) == 0)
		{
			timeout = atoi(&argv[i][2]);
			if (timeout > 30000)
				timeout = 1000;
		}
	}

	printf("Retrieving names for device %s timeout:%d...\n", argv[0], timeout);

	nResult = CprGetDeviceAndPortNames(argv[0], devName, port1Name, port2Name, &nameLen, timeout);
	if (nResult == CPR_OK)
	{
		printf(" Device Name     : %s\n", devName);
		printf(" Tcp Port 1 Name : %s\n", port1Name);
		printf(" Tcp Port 2 Name : %s\n", port2Name);
	}
	else
	{
		PrintError("Error calling GetDeviceAndPortNames", nResult);
	}

	return 0;
}


int CmdSetPortInfo(int argc, char* argv[])
{
	int			nResult;
	int			a = -1;
	int			r = -1;
	int			c = -1;
	int			d = -1;
	int			u = -1;
	int			b = -1;
	int			t = -1;
	int			L = -1;
	int			k = -1;
	int			m = -1;
	int			i = -1;
	int			p = -1;
	int			s = -1;
	PortInfo	portInfo;
	int			portNum;
	BOOL		abort = FALSE;

	if (argc < 2)
	{
		printf("Please provide a port number and at least one other argument.\n");
		DISPLAY_HELP("setinfo");
		return 0;
	}

	portNum = atoi(argv[0]);

	ParsePortInfo(argc-1, &argv[1], &a, &r, &c, &d, &u, &b, &t, &L, &k, &m, &i, &p, &s);

    nResult = CprGetPortInfo(&portInfo, portNum);

	if (nResult == CPR_OK)
	{
		printf("Before: \n");
		LogPortInfo(&portInfo);
	}
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
		return 0;
	}
	else
	{
		PrintError("Error calling GetPortInfo", nResult);
		return 0;
	}


	if (a != -1)
		portInfo.AutoReconErr = ZERO_OR_ONE(a);
	if (s != -1)
		portInfo.AutoReconPeer = ZERO_OR_ONE(s);
	if (r != -1)
		portInfo.ReconLimit = r;
	if (c != -1)
		portInfo.CnTmo = c;
	if (d != -1)
	{
		if ((d < 0) || (d > 5))
		{
			printf("  DTR connection value '%d' is invalid. 0 to 5 only.\n", d);
			abort = TRUE;
		}
		portInfo.DtrConnection = d;
	}
	if (u != -1)
		portInfo.UseRFC2217 = ZERO_OR_ONE(u);
	if (b != -1)
		portInfo.BufferWrites = ZERO_OR_ONE(b);
	if (t != -1)
		portInfo.TransmitEmpty = ZERO_OR_ONE(t);
	if (L != -1)
	{
		if ((L < 0) || (L > 2))
		{
			printf("  Listen Mode value '%d' is invalid. 0, 1 or 2 only.\n", L);
			abort = TRUE;
		}
		portInfo.ListenMode = L;
	}
	if (k != -1)
		portInfo.KeepAliveOnOff = ZERO_OR_ONE(k);
	if (m != -1)
		portInfo.KeepAliveTime = m;
	if (i != -1)
		portInfo.KeepAliveInterval = i;
	if ((p != -1) && (portInfo.ListenMode))
	{
		portInfo.Services[0].Port = p;
		portInfo.NumServices = 1;
	}

	if (abort)
	{
		DISPLAY_HELP("setinfo");
	}
	else
	{
		// port number is used from portInfo.portNumber in SetPortInfo
		//
		nResult = CprSetPortInfo(&portInfo);
		if (nResult == CPR_OK)
		{
			printf("\nAfter: \n");
			LogPortInfo(&portInfo);
		}
		else if (nResult == 0x57)
		{
			printf("  This is not a CPR port\n");
		}
		else
		{
			PrintError("Error calling SetPortInfo", nResult);
		}
	}

	return 0;
}

int CmdSetDefPortInfo(int argc, char* argv[])
{
	int			nResult;
	int			a = -1;
	int			r = -1;
	int			c = -1;
	int			d = -1;
	int			u = -1;
	int			b = -1;
	int			t = -1;
	int			L = -1;
	int			k = -1;
	int			m = -1;
	int			i = -1;
	int			p = -1;
	int			s = -1;
	PortInfo	portInfo;
	BOOL		abort = FALSE;

	if (argc == 0)
	{
		printf("Please provide at least one argument.\n");
		DISPLAY_HELP("setdefinfo");
		return 0;
	}

	ParsePortInfo(argc-1, &argv[1], &a, &r, &c, &d, &u, &b, &t, &L, &k, &m, &i, &p, &s);

	if (a != -1)
		portInfo.AutoReconErr = ZERO_OR_ONE(a);
	if (s != -1)
		portInfo.AutoReconPeer = ZERO_OR_ONE(s);
	if (r != -1)
		portInfo.ReconLimit = r;
	if (c != -1)
		portInfo.CnTmo = c;
	if (d != -1)
	{
		if ((d < 0) || (d > 5))
		{
			printf("  DTR connection value '%d' is invalid. 0 to 5 only.\n", d);
			abort = TRUE;
		}
		portInfo.DtrConnection = d;
	}
	if (u != -1)
		portInfo.UseRFC2217 = ZERO_OR_ONE(u);
	if (b != -1)
		portInfo.BufferWrites = ZERO_OR_ONE(b);
	if (t != -1)
		portInfo.TransmitEmpty = ZERO_OR_ONE(t);
	if (L != -1)
	{
		if ((L < 0) || (L > 2))
		{
			printf("  Listen Mode value '%d' is invalid. 0, 1 or 2 only.\n", L);
			abort = TRUE;
		}
		portInfo.ListenMode = L;
	}
	if (k != -1)
		portInfo.KeepAliveOnOff = ZERO_OR_ONE(k);
	if (m != -1)
		portInfo.KeepAliveTime = m;
	if (i != -1)
		portInfo.KeepAliveInterval = i;
	if ((p != -1) && (portInfo.KeepAliveOnOff))
	{
		portInfo.Services[0].Port = p;
		portInfo.NumServices = 1;
	}

	if (abort)
	{
		DISPLAY_HELP("setinfo");
	}
	else
	{
		nResult = CprGetDefaultPortInfo(&portInfo);
		if (nResult == CPR_OK)
		{
			printf("Before: \n");
			LogPortInfo(&portInfo);
		}
		else
		{
			PrintError("Error calling GetDefaultPortInfo", nResult);
			return 0;
		}

		// port number is used from portInfo.portNumber in SetPortInfo
		//
		nResult = CprSetDefaultPortInfo(&portInfo);
		if (nResult == CPR_OK)
		{
			printf("\nAfter: \n");
			LogPortInfo(&portInfo);
		}
		else
		{
			PrintError("Error calling SetDefaultPortInfo", nResult);
		}
	}

	return 0;
}

void ParsePortInfo(int argc, char *argv[], int *a, int *r, int *c, int *d, int *u, int *b, int *t, int *L, int *k, int *m, int *it, int *p, int *s)
{
	int i;

	for (i = 0; i < argc; i++)
	{
		if (_strnicmp("-a", argv[i], 2) == 0)
			*a = atoi(&argv[i][2]);
		else if (_strnicmp("-r", argv[i], 2) == 0)
			*r = atoi(&argv[i][2]);
		else if (_strnicmp("-c", argv[i], 2) == 0)
			*c = atoi(&argv[i][2]);
		else if (_strnicmp("-d", argv[i], 2) == 0)
			*d = atoi(&argv[i][2]);
		else if (_strnicmp("-u", argv[i], 2) == 0)
			*u = atoi(&argv[i][2]);
		else if (_strnicmp("-b", argv[i], 2) == 0)
			*b = atoi(&argv[i][2]);
		else if (_strnicmp("-t", argv[i], 2) == 0)
			*t = atoi(&argv[i][2]);
		else if (_strnicmp("-L", argv[i], 2) == 0)
			*L = atoi(&argv[i][2]);
		else if (_strnicmp("-k", argv[i], 2) == 0)
			*k = atoi(&argv[i][2]);
		else if (_strnicmp("-m", argv[i], 2) == 0)
			*m = atoi(&argv[i][2]);
		else if (_strnicmp("-i", argv[i], 2) == 0)
			*it = atoi(&argv[i][2]);
		else if (_strnicmp("-p", argv[i], 2) == 0)
			*p = atoi(&argv[i][2]);
		else if (_strnicmp("-s", argv[i], 2) == 0)
			*s = atoi(&argv[i][2]);
	}
}

int CmdAddService(int argc, char* argv[])
{
	int			nResult;
	PortInfo	portInfo;
	int			portNum;
	char		*ipAddr;
	int			tcpPort;
	DWORD		i;

	if (argc < 3)
	{
		DISPLAY_HELP("addservice");
		return 0;
	}

	portNum = atoi(argv[0]);
	ipAddr = argv[1];
	tcpPort = atoi(argv[2]);

	if (strlen(ipAddr) > 15)
	{
		printf("Invalid IP Address: %s\n", ipAddr);
	}

    nResult = CprGetPortInfo(&portInfo, portNum);
    if (nResult == CPR_OK)
    {
		printf("Before: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
		return 0;
	}
    else
    {
        PrintError("Error calling GetPortInfo", nResult);
		return 0;
    }

	if (portInfo.NumServices == MAX_SERVICES)
	{
		printf("\nThe max services, %d, has already been reached. Service not added.\n", MAX_SERVICES);
		return 0;
	}

	portInfo.Services[portInfo.NumServices].Port = tcpPort;
	for (i = 0; i < strlen(ipAddr); i++)
		portInfo.Services[portInfo.NumServices].Address[i] = ipAddr[i];
	portInfo.NumServices++;

	// port number is used from portInfo.portNumber in SetPortInfo
	//
    nResult = CprSetPortInfo(&portInfo);
    if (nResult == CPR_OK)
    {
		printf("\nAfter: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling SetPortInfo", nResult);
    }

	return 0;
}

int CmdRemoveService(int argc, char* argv[])
{
	int			nResult;
	PortInfo	portInfo;
	DWORD		serviceNum;
	int			portNum;
	DWORD		i;

	if (argc < 2)
	{
		DISPLAY_HELP("remservice");
		return 0;
	}

	portNum = atoi(argv[0]);
	serviceNum = atoi(argv[1]);

    nResult = CprGetPortInfo(&portInfo, portNum);
    if (nResult == CPR_OK)
    {
		printf("Before: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
		return 0;
	}
    else
    {
        PrintError("Error calling GetPortInfo", nResult);
		return 0;
    }

	if (portInfo.NumServices == 0)
	{
		printf("\nThere are no services to remove\n");
		return 0;
	}
	if (serviceNum > portInfo.NumServices)
	{
		printf("\nInvalid Service Index must be from 1 to %d.\n", portInfo.NumServices);
		return 0;
	}

	portInfo.NumServices--;
	for (i = serviceNum-1; i < portInfo.NumServices; i++)
	{
		portInfo.Services[i] = portInfo.Services[i+1];
	}

	// port number is used from portInfo.portNumber in SetPortInfo
	//
    nResult = CprSetPortInfo(&portInfo);
    if (nResult == CPR_OK)
    {
		printf("\nAfter: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling SetPortInfo", nResult);
    }

	return 0;
}

int CmdSetService(int argc, char* argv[])
{
	int			nResult;
	PortInfo	portInfo;
	DWORD		serviceNum;
	int			portNum;
	DWORD		i;
	char		*ipAddr;
	int			tcpPort;

	if (argc < 4)
	{
		DISPLAY_HELP("setservice");
		return 0;
	}

	portNum = atoi(argv[0]);
	serviceNum = atoi(argv[1]);
	ipAddr = argv[2];
	tcpPort = atoi(argv[3]);

    nResult = CprGetPortInfo(&portInfo, portNum);
    if (nResult == CPR_OK)
    {
		printf("Before: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
		return 0;
	}
    else
    {
        PrintError("Error calling GetPortInfo", nResult);
		return 0;
    }

	if (portInfo.NumServices == 0)
	{
		printf("\nThere are no services to remove\n");
		return 0;
	}
	if (serviceNum > portInfo.NumServices)
	{
		printf("\nInvalid Service Index must be from 1 to %d.\n", portInfo.NumServices);
		return 0;
	}

	portInfo.Services[serviceNum].Port = tcpPort;
	for (i = 0; i < strlen(ipAddr); i++)
		portInfo.Services[serviceNum].Address[i] = ipAddr[i];

    nResult = CprSetPortInfo(&portInfo);
    if (nResult == CPR_OK)
    {
		printf("\nAfter: \n");
		LogCprServices(&portInfo);
    }
	else if (nResult == 0x57)
	{
		printf("  This is not a CPR port\n");
	}
    else
    {
        PrintError("Error calling SetPortInfo", nResult);
    }

	return 0;
}

int CmdImportCPC(int argc, char* argv[])
{
	int			nResult;
	char		*cpcFileName;
	PortInfo	*pPortInfo = NULL;
	DWORD		i;
	int			j;
	DWORD		portCnt;
	BOOL		print = TRUE;
	BOOL		addPorts = FALSE;
	BOOL		prompt = TRUE;
    PortMap		portMap;

	if (argc < 1)
	{
		printf("Please provide a Com Port Config file name\n");
		DISPLAY_HELP("importcpc");
		return 0;
	}

	for (j = 0; j < argc; j++)
	{
		if (_strnicmp("-s", argv[j], 2) == 0)
			print = FALSE;
		else if (_strnicmp("-a", argv[j], 2) == 0)
			addPorts = TRUE;
		else if (_strnicmp("-n", argv[j], 2) == 0)
			prompt = FALSE;
	}

	cpcFileName = argv[0];

	portCnt = MAX_PORTS;
	printf("Importing        : ");
	nResult = CprImportComPortConfigFile(cpcFileName, (PPortInfo*)&pPortInfo, &portCnt);

	do
	{
		if (nResult == CPR_OK)
		{
			printf("%d ports\n\n", portCnt);

			if (print || addPorts)
			{
				if (addPorts)
				{
					if (prompt)
					{
						printf("Are you sure you want to remove old ports and add new ports? (Y or N)? ");
						if (toupper(_getch()) != 'Y')
						{
							if (print)
								printf("\nUser cancelled removal\n");
							break;
						}
						printf("\n");
					}

					// Remove old ports first:
					nResult = CprGetPortMap(&portMap);
					if (nResult != CPR_OK)
					{
						printf("\nUnable to retrieve Port Map\n");
						break;
					}
				
					for (i = 0; i < portMap.numPorts; i++)
					{
						if (print)
							printf("Removing Port %-3d : ", portMap.Map[i].portNumber);

						nResult = CprRemovePort(portMap.Map[i].portNumber);
						if (nResult == CPR_OK)
						{
							if (print)
								printf("Success\n");
						}
						else
						{
							if (print)
								printf("not removed. An error occurred while attempting to remove this port.\n");
						}
					}
				}

				for (i = 0; i < portCnt; i++)
				{
					if (addPorts)
					{
						if (print)
							printf("Adding Port %-3d   : ", pPortInfo[i].portNumber);
						nResult = CprAddPort(pPortInfo[i].portNumber, FALSE);
						if (nResult == CPR_OK)
						{
							if (print)
								printf("Success\n");
						}
						else if (nResult == 0x57)
						{
							if (print)
								printf("This is not a CPR port\n");
						}
						else
						{
							printf("Error adding Port\n");
						}

						nResult = CprSetPortInfo(&pPortInfo[i]);
						if (nResult == CPR_OK)
						{
							if (print)
								printf("PortInfo          : Updated Successfully\n");
						}
						else if (nResult == 0x57)
						{
							if (print)
								printf("This is not a CPR port\n");
						}
						else
						{
							printf("Error updating PortInfo for Port %d", pPortInfo[i].portNumber);
						}
					}
					if (print)
					{
						swprintf(pPortInfo[i].portName, 8, L"COM%d", pPortInfo[i].portNumber);
						LogPortInfo(&pPortInfo[i]);
						printf("\n");
					}
				}
			}
		}
		else
		{
			PrintError("Error calling ImportComPortConfigFile", nResult);
		}
	} while (FALSE);

	if (pPortInfo)
	{
		////////////////////////////////////////////////////////////
		// FreeBuffer
		////////////////////////////////////////////////////////////
		nResult = CprFreeBuffer((PVOID)pPortInfo);

		if (nResult != CPR_OK)
		{
	        PrintError("Error calling FreeBuffer.", nResult);
		}
	}

	return 0;
}

int CmdExportCPC(int argc, char* argv[])
{
	int			nResult;
	char		*cpcFileName;
	PortInfo	pPortInfo[MAX_PORTS];
	DWORD		i;
	DWORD		portCnt;
    PortMap		portMap;

	if (argc < 1)
	{
		printf("Please provide a Com Port Config file name\n");
		DISPLAY_HELP("exportcpc");
		return 0;
	}

	cpcFileName = argv[0];

	nResult = CprGetPortMap(&portMap);
    if (nResult != CPR_OK)
    {
		printf("Unable to retrieve Port Map\n");
		return 0;
	}

	portCnt = 0;
	for (i = 0; i < portMap.numPorts; i++)
	{
	    nResult = CprGetPortInfo(&pPortInfo[portCnt], portMap.Map[i].portNumber);
	    if (nResult == CPR_OK)
			portCnt++;
	}

	if (portCnt == 0)
	{
		printf("No CPR ports availabale for Export\n");
		return 0;
	}

	printf("%d ports\n", portCnt);

	nResult = CprExportComPortConfigFile(cpcFileName, pPortInfo, portCnt);
    if (nResult == CPR_OK)
    {
		printf("\nExport was successful\n");
    }
    else
    {
        PrintError("Error calling ExportComPortConfigFile", nResult);
    }

	return 0;
}

int CmdRemoveAllPorts(int argc, char* argv[])
{
	int nResult;
	int	j;
	BOOL print = TRUE;
	BOOL prompt = TRUE;
    PortMap	portMap;
	DWORD	i;
	
	for (j = 0; j < argc; j++)
	{
		if (_strnicmp("-s", argv[j], 2) == 0)
			print = FALSE;
		else if (_strnicmp("-n", argv[j], 2) == 0)
			prompt = FALSE;
	}

	if (prompt)
	{
		printf("Are you sure you want to remove ALL the CPR ports (Y or N)? ");
		if (toupper(_getch()) != 'Y')
		{
			if (print)
				printf("\nUser cancelled removal\n");
			return 0;
		}
		printf("\n");
	}

	nResult = CprGetPortMap(&portMap);
    if (nResult != CPR_OK)
    {
		printf("Unable to retrieve Port Map\n");
		return 0;
	}

	for (i = 0; i < portMap.numPorts; i++)
	{
		if (print)
			printf("Port %-3d: ", portMap.Map[i].portNumber);

		nResult = CprRemovePort(portMap.Map[i].portNumber);
		if (nResult == CPR_OK)
		{
			if (print)
				printf("removed successfully\n");
		}
		else
		{
			if (print)
				printf("not removed. An error occurred while attempting to remove this port.\n");
		}
	}

	return 0;
}

void Sort(PortMap *pPortMap)
{
	// Slow sort, but quick enough for such a small data set.
	ULONG tmp;
	ULONG i;
	ULONG n;
	ULONG inner = pPortMap->numPorts - 1;

	for (i = 0; i < pPortMap->numPorts - 1; i++)
	{
		for (n = 0; n < inner; n++)
		{
			if (pPortMap->Map[n].portNumber > pPortMap->Map[n+1].portNumber)
			{
				tmp = pPortMap->Map[n].portNumber;
				pPortMap->Map[n].portNumber = pPortMap->Map[n+1].portNumber;
				pPortMap->Map[n+1].portNumber = tmp;
			}
		}
		inner--;
	}
}

int CmdLoopback(int argc, char* argv[])
{
	int nResult;
	int	i, j;
	DWORD ndx;
	int numThreads = 1;
	BOOL random = FALSE;
	int dataSize = 100;
	int numDataRuns = 1;
	int comPorts[MAX_PORTS];
	int numComPorts = 0;
	char *comPortsStr = "ALL";
	int printInterval = 10;
	BOOL printErrors = FALSE;
    PortMap	portMap;
	BOOL verbose = FALSE;
	int numRuns = 0;

	for (i = 0; i < argc; i++)
	{
		if (_strnicmp("-t", argv[i], 2) == 0)
			numThreads = atoi(&argv[i][2]);
		else if (_strnicmp("-r", argv[i], 2) == 0)
			random = TRUE;
		else if (_strnicmp("-d", argv[i], 2) == 0)
			dataSize = atoi(&argv[i][2]);
		else if (_strnicmp("-e", argv[i], 2) == 0)
			printErrors = (atoi(&argv[i][2])) ? TRUE : FALSE;
		else if (_strnicmp("-n", argv[i], 2) == 0)
			numDataRuns = atoi(&argv[i][2]);
		else if (_strnicmp("-p", argv[i], 2) == 0)
			printInterval = atoi(&argv[i][2]);
		else if (_strnicmp("-x", argv[i], 2) == 0)
			numRuns = atoi(&argv[i][2]);
		else if (_strnicmp("-v", argv[i], 2) == 0)
			verbose = TRUE;
		else if (_strnicmp("-c", argv[i], 2) == 0)
		{
			comPortsStr = &argv[i][2];

            char *next_token;
			char *token = strtok_s(comPortsStr, ",", &next_token);

			while (token)
			{
				char *dash = strchr(token, '-');
				if (dash)
				{
					*dash = '\0';
					int firstCp = atoi(token);
					int lastCp = atoi(dash+1);
					if (firstCp < lastCp)
					{
						for (j = firstCp; j <= lastCp; j++)
						{
							comPorts[numComPorts++] = j;
						}
					}
				}
				else
				{
					comPorts[numComPorts++] = atoi(token);
				}

				token = strtok_s(NULL, ",", &next_token);
			}
		}
	}

	if (numComPorts == 0)
	{
		nResult = CprGetPortMap(&portMap);
		if (nResult != CPR_OK)
		{
			printf("Unable to retrieve Port Map\n");
			return 0;
		}

		Sort(&portMap);

		for (ndx = 0; ndx < portMap.numPorts; ndx++)
		{
			comPorts[numComPorts++] = portMap.Map[ndx].portNumber;
		}
	}

	if (numThreads == 0) numThreads = 1;
	if (numComPorts == 0) numComPorts = 1;
	if (numThreads > numComPorts) numThreads = numComPorts;

	return LaunchThreads(
		numThreads,
		random,
		printErrors,
		dataSize,
		numDataRuns,
		comPorts,
		numComPorts,
		printInterval,
		numRuns,
		verbose);
}

int CmdReleasePort(int argc, char* argv[])
{
	if (argc == 1)
	{
		DWORD port = atoi(argv[0]);
		int status = CprReleasePort(port);
		if (status == CPR_OK)
			printf("Release was successful\n");
		else
	        PrintError("Error calling CprReleasePort.", status);
	}

	return 0;
}
