//
// CprMgrCmd.cpp : Defines the entry point for the console application.
//
// Must compile using UNICODE and _UNICODE preprocessor defines.
//
#include "stdafx.h"

char cmdLine[MAX_CHARS];
char history[MAX_HISTORY][MAX_CHARS];
int curHist;
char prompt[MAX_PROMPT];
BOOL insertMode = TRUE;

int _tmain(int argc, _TCHAR* argv[])
{
	int i;
	CHAR appPath[MAX_PATH];

	//strcpy(prompt, ">");
	strcpy_s(prompt, MAX_PROMPT, ">");
	curHist = 0;
	cmdLine[0] = '\0';
	for (i = 0; i < MAX_HISTORY; i++)
		history[i][0] = '\0';

	if (argc > 1)
	{
		if (_stricmp(argv[1], "DIR") == 0)
		{
			if (GetCurrentDirectory(sizeof(appPath), appPath) != 0)
			{
				//
				// The driver directory with the .INF and .SYS files
				// need to below this directory.
				// For example if appPath = C:\MyApp\
				//     then the directory C:\MyApp\driver must exist
				//     and the file C:\MyApp\driver\CprDrvr.inf must exist
				//     and the file C:\MyApp\driver\CprDrvr.sys must exist
				//
				//strcat(appPath, "\\");
				strcat_s(appPath, MAX_PATH, "\\");
				CprRegisterApplicationPath( appPath );
			}
		}
	}

	return CommandLoop(argc, argv);
}

int CommandLoop(int argc, _TCHAR* argv[])
{
	PCommandItem cmdItem = NULL;
	char *cmdArgv[MaxArgs];
	int cmdArgc;
	int ndx;

	if ((argc > 1) && _stricmp(argv[1], "DIR"))
	{
		// one shot command line mode

		// find command
		for (ndx = 1; ndx < NumCmds; ndx++)
		{
			if (_strcmpi(argv[1], Commands[ndx].command) == 0)
			{
				cmdItem = &Commands[ndx];
				break;
			}
		}

		// If command not found, use Unknown command
		if (cmdItem == NULL)
		{
			cmdItem = &Commands[0];
			argc = 1;
		}
		else
			argc-=2;

		if (cmdItem != NULL)
		{
			int retval = cmdItem->function(argc, &argv[2]);
			if (retval != 0)
			{
				printf("\nAn error ocurred: %d. Exiting.\n", retval);
				return retval;
			}
		}
		else
		{
			printf("\nError in Command Line Parsing");
			return -1;
		}
	}
	else
	{
		while (TRUE)
		{
			if (GetCommandItem(&cmdItem, &cmdArgc, cmdArgv) != 0)
				break;

			if (cmdItem != NULL)
			{
				int retval = cmdItem->function(cmdArgc, cmdArgv);
				if (retval != 0)
				{
					printf("\nAn error ocurred: %d. Exiting.\n", retval);
					break;
				}
			}
			else
			{
				printf("\nError in Command Line Parsing");
			}
		}
	}

	return 0;
}

int GetCommandItem(PCommandItem *ppCmdItem, int *pArgc, char* argv[])
{
	int retval = 0;
	int tc = 0;
	int ndx = 0;
	int end = 0;
	int pos = 0;
	int endSave;
	int i;
	char *cmd;
	char *prevCmd = NULL;
	int tmpHist = curHist;
	
	*ppCmdItem = NULL;
	*pArgc = 0;

	while (true)
	{
		PrintPrompt();

		while (true)
		{
			// get command from user
			//
			tc = _getch();

			if (isprint(tc) && tc != ' ')
			{
				// We have a printable character
				// Do not handle spaces here, this character
				// can be used in Command Completion
				//
				if ((pos == end) || ((pos < end) && !insertMode))
				{
					// Either at end or Overwrite Mode
					_putch(tc);
					if (pos == end)
						end++;
					cmdLine[pos++] = tc;
				}
				else
				{
					// Insert Mode
					int k;
					memmove(&cmdLine[pos+1], &cmdLine[pos], end-pos);
					cmdLine[pos++] = tc;
					_putch(tc);
					end++;
					for (k = pos; k < end; k++)
						_putch(cmdLine[k]);
					for (k = pos; k < end; k++)
						_putch('\b');
				}

				// Allow a single ? for help to break out without a return
				//
				if ((end == 1) && ((tc == '?') || (tc == '/')))
					break;
			}
			else if ((tc == '\r') || (tc == '\t') || (tc == ' '))
			{
				// Return, Tab or Space
				//
				endSave = end;
				if (CompleteCommand(cmdLine, &end))
				{
					for (i = endSave; i < end; i++)
					{
						_putch(cmdLine[i]);
						pos = end;
					}

					if (tc == '\t')
					{
						_putch(' ');
						cmdLine[pos++] = ' ';
						end++;
//						pos++;
					}
				}

				if ((pos != 0) && (tc == ' '))
				{
					_putch(tc);
					cmdLine[pos++] = tc;
					end++;
				}

				if (tc == '\r')
					break;
			}
			else if (tc == 0x3)
			{
				// Ctrl-C
				//
				printf("\n\nTerminated by user via Ctrl-C\n\n");
				return -1;
			}
			else if (tc == '\b')
			{
				// Backspace
				//
				if (pos > 0)
				{
					if (pos == end)
					{
						_putch(tc);
						_putch(' ');
						_putch(tc);
					}
					else
					{
						int k;
						_putch(tc);
						strcpy_s(&cmdLine[pos-1], MAX_CHARS, &cmdLine[pos]);
						for (k = pos-1; k < end-1; k++)
							_putch(cmdLine[k]);
						_putch(' ');
						for (k = pos-1; k < end; k++)
							_putch('\b');
					}
					pos--;
					end--;
				}
			}
			else if (tc == 0xE0)
			{
				// Special Keys
				//
				tc = _getch();

				if (tc == 'H')
				{
					// Up Arrow
					if (tmpHist != 0)
					{
						ClearLine(&end);
						tmpHist--;
						PrintHist(&end, tmpHist);
						pos = end;
					}
				}
				else if (tc == 'P')
				{
					// Down Arrow
					if (tmpHist < MAX_HISTORY)
					{
						if (history[tmpHist][0] != 0)
						{
							tmpHist++;
							if (tmpHist < MAX_HISTORY)
							{
								ClearLine(&end);
								PrintHist(&end, tmpHist);
								pos = ndx;
							}
							else
								tmpHist = MAX_HISTORY - 1;
						}
					}
				}
				else if (tc == 'K')
				{
					// Left Arrow
					if (pos-1 >= 0)
					{
						pos--;
						_putch('\b');
					}
				}
				else if (tc == 'M')
				{
					// Right Arrow
					if (pos + 1 <= end)
					{
						_putch(cmdLine[pos]);
						pos++;
					}
				}
				else if (tc == 'S')
				{
					// Delete Key
					if (end != pos)
					{
						int k;
						strcpy_s(&cmdLine[pos], MAX_CHARS, &cmdLine[pos+1]);
						for (k = pos; k < end; k++)
							_putch(cmdLine[k]);
						_putch(' ');
						for (k = pos; k < end+1; k++)
							_putch('\b');
						end--;
					}
				}
				else if (tc == 'R')
				{
					// Toggle Insert Mode
					insertMode = !insertMode;
				}
			}
		}

		if ((end == 0) && (tc == '\r'))
		{
			end = 0;
		}
		else
			break;
	}

	// Reached the end of history
	// remove first half of history
	if (curHist == MAX_HISTORY)
	{
		int half = curHist / 2;
		for (i = 0; i < half; i++)
		{
			strcpy_s(history[i], MAX_CHARS, history[i + half]);
			history[i + half][0] = '\0';
		}
		curHist = half;
	}

	// Terminate command string
	cmdLine[end] = '\0';
	cmd = cmdLine;

	// Copy to history buffer
	strcpy_s(history[curHist++], MAX_CHARS, cmdLine);

	// Trim spaces off the end.
	for (i = end-1; i >= 0; i--)
	{
		if (cmdLine[i] == ' ')
			cmdLine[i] = '\0';
		else
			break;
	}

	ParseString(cmdLine, FALSE, end, ' ', pArgc, argv, MaxArgs);

	// find command
	for (ndx = 1; ndx < NumCmds; ndx++)
	{
		if (_strcmpi(cmd, Commands[ndx].command) == 0)
		{
			*ppCmdItem = &Commands[ndx];
			break;
		}
	}

	// find hidden command
	for (ndx = 0; ndx < NumHiddenCmds; ndx++)
	{
		if (_strcmpi(cmd, HiddenCommands[ndx].command) == 0)
		{
			*ppCmdItem = &HiddenCommands[ndx];
			break;
		}
	}

	// If command not found, use Unknown command
	if (*ppCmdItem == NULL)
	{
		*ppCmdItem = &Commands[0];
		*pArgc = 1;
		argv[0] = cmd;
	}

	printf("\n");

	return retval;
}

void ParseString(char *str, BOOL useFirst, int end, char sep, int *pArgc, char *argv[], int maxArgs)
{
	int		i;
	char	*prevCmd = NULL;
	BOOL	foundQuote = FALSE;

	*pArgc = 0;

	// parse command string and separate arguments
	for (i = 0; i < end-1; i++)
	{
		if (str[i] == '\"')
		{
			foundQuote = !foundQuote;
		}
		else if ((str[i] == sep) && !foundQuote)
		{
			str[i] = '\0';

			if (prevCmd == NULL)
			{
				if (useFirst)
				{
					argv[*pArgc] = str;
					(*pArgc)++;
				}
			}
			else
			{
				if (*pArgc >= maxArgs-1)
					break;

				argv[*pArgc] = prevCmd;
				(*pArgc)++;
			}

			prevCmd = &str[i+1];
		}
	}

	// Now pick up last argument
	if (useFirst && (*pArgc == 0))
	{
		argv[*pArgc] = str;
		(*pArgc)++;
	}
	else if (prevCmd)
	{
		argv[*pArgc] = prevCmd;
		(*pArgc)++;
	}
}

void PrintPrompt()
{
	printf("\n%s", prompt);
}


BOOL CompleteCommand(char *cmd, int *ndx)
{
	int i;
	BOOL retval = FALSE;

	for (i = 1; i < NumCmds; i++)
	{
		if (Commands[i].uniqueCnt <= *ndx)
		{
			if (_strnicmp(cmd, Commands[i].command, *ndx) == 0)
			{
				// Found it
				*ndx = (int)strlen(Commands[i].command);
				strcpy_s(cmd, MAX_CHARS, Commands[i].command);
				retval = TRUE;
				break;
			}
		}
	}

	return retval;
}

void ClearLine(int *pNdx)
{
	int i;

	for (i = 0; i < *pNdx; i++)
	{
		_putch('\b');
		_putch(' ');
		_putch('\b');
	}

	cmdLine[0] = '\0';
	*pNdx = 0;
}

void PrintHist(int *pNdx, int hist)
{
	int i;

	for (i = 0; i < MAX_CHARS; i++)
	{
		if (history[hist][i] == '\0')
			break;
		cmdLine[i] = history[hist][i];
		_putch(history[hist][i]);
	}

	*pNdx = i;
}

void PrintError(char *str, DWORD dwError)
{
	char	str1[512];
	char	str2[512];

	if (GetErrorString(str1, 512, dwError) > 0)
	{
		sprintf_s(str2, 512, "%s : %d (0x%x) [%s]\n", str, dwError, dwError, str1);
	}
	else
	{
		sprintf_s(str2, 512, "%s : %d (0x%x)\n", str, dwError, dwError);
	}

    printf(str2);
}

DWORD GetErrorString(char *buf, DWORD bufSize, DWORD dwError)
{
	DWORD retval;
	DWORD i;
	char *pStr = "Unknown";
	BOOL  cprFound = TRUE;

	switch (dwError)
	{
		case (DWORD)CPR_INVALID_ARGUMENT:			pStr = "CPR_INVALID_ARGUMENT";				break;
		case (DWORD)CPR_UNKNOWN_EXCEPTION:			pStr = "CPR_UNKNOWN_EXCEPTION";				break;
		case (DWORD)CPR_INSUFFICIENT_RESOURCES:		pStr = "CPR_INSUFFICIENT_RESOURCES";		break;
		case (DWORD)CPR_NO_ADAPTERS_FOUND:			pStr = "CPR_NO_ADAPTERS_FOUND";				break;
		case (DWORD)CPR_NO_DEVICES_FOUND:			pStr = "CPR_NO_DEVICES_FOUND";				break;
		case (DWORD)CPR_NO_DRIVER_FOUND:			pStr = "CPR_NO_DRIVER_FOUND";				break;
		case (DWORD)CPR_PORT_NOT_FOUND:				pStr = "CPR_PORT_NOT_FOUND";				break;
		case (DWORD)CPR_PORT_KEY_NOT_FOUND:			pStr = "CPR_PORT_KEY_NOT_FOUND";			break;
		case (DWORD)CPR_NO_DEVICE_PARAMETERS_FOUND:	pStr = "CPR_NO_DEVICE_PARAMETERS_FOUND";	break;
		case (DWORD)CPR_PORT_NAME_NOT_FOUND:		pStr = "CPR_PORT_NAME_NOT_FOUND";			break;
		case (DWORD)CPR_EXT_PORT_INFO_NOT_FOUND:	pStr = "CPR_EXT_PORT_INFO_NOT_FOUND";		break;
		case (DWORD)CPR_REGISTRY_SAVE_ERROR:		pStr = "CPR_REGISTRY_SAVE_ERROR";			break;
		case (DWORD)CPR_DRIVER_OPEN_FAILED:			pStr = "CPR_DRIVER_OPEN_FAILED";			break;
		case (DWORD)CPR_DRIVER_COMM_FAILED:			pStr = "CPR_DRIVER_COMM_FAILED";			break;
		case (DWORD)CPR_DRIVER_CLOSE_FAILED:		pStr = "CPR_DRIVER_CLOSE_FAILED";			break;
		case (DWORD)CPR_NO_EVENT_LOG:				pStr = "CPR_NO_EVENT_LOG";					break;
		case (DWORD)CPR_BUFFER_TOO_SMALL:			pStr = "CPR_BUFFER_TOO_SMALL";				break;
		case (DWORD)CPR_DEVICE_NOT_FOUND:			pStr = "CPR_DEVICE_NOT_FOUND";				break;
		case (DWORD)CPR_CPC_FILE_OPEN_ERROR:		pStr = "CPR_CPC_FILE_OPEN_ERROR";			break;
		case (DWORD)CPR_CPC_INVALID_CPC_FILE:		pStr = "CPR_CPC_INVALID_CPC_FILE";			break;
		case (DWORD)CPR_CPC_FILE_WRITE_ERROR:		pStr = "CPR_CPC_FILE_WRITE_ERROR";			break;
		case (DWORD)CPR_CPC_FILE_READ_ERROR:		pStr = "CPR_CPC_FILE_READ_ERROR";			break;
		case (DWORD)CPR_CPC_FILE_PARSE_ERROR:		pStr = "CPR_CPC_FILE_PARSE_ERROR";			break;
		case (DWORD)CPR_INVALID_RANGE:				pStr = "CPR_INVALID_RANGE";					break;
		case (DWORD)CPR_PORT_ALREADY_EXISTS:		pStr = "CPR_PORT_ALREADY_EXISTS";			break;
		case (DWORD)CPR_INVALID_PORT_NUMBER:		pStr = "CPR_INVALID_PORT_NUMBER";			break;
		default: cprFound = FALSE; break;
	}

	if (cprFound == FALSE)
	{
		retval = FormatMessage(
					FORMAT_MESSAGE_FROM_SYSTEM,
					NULL,
					dwError,
					0,		//MAKELANGID( LANG_ENGLISH, LANG_NEUTRAL ),
					buf,
					bufSize,
					NULL );                                   

		if (retval > 0)
		{
			// Don't want the \n at the end of the string
			for (i = 0; i < bufSize; i++)
			{
				if (buf[i] == '\0')
					break;
			}

			if ((i < bufSize) && (i != 0) && (buf[i-1] == '\n'))
			{
				buf[i-1] = '\0';
			}
		}
	}
	else
	{
		sprintf_s(buf, bufSize, "%s", pStr);
		retval = 1;
	}

	return retval;
}
