// CprMgrCmd.h
//

#ifndef _CPRMGRCMD_H_
#define _CPRMGRCMD_H_

//
// Constants
//
#define TO_BOOL(v)		((v) ? "True" : "False")
#define ZERO_OR_ONE(v)	((v==0) ? 0 : 1)
#define NumCmds			33
#define NumHiddenCmds	1
#define MaxArgs			20
#define MAX_CHARS		256
#define MAX_HISTORY		100
#define MAX_PROMPT		20

//
// Structures
//
typedef
int
(*PCOMMAND_FUNCTION) (int argc, char* argv[]);

typedef struct _Arg
{
	char *arg;
	char *usage;
} Arg, *PArg;

typedef struct _CommandArgs
{
	int numArgs;
	PArg args;
} CommandArgs, *PCommandArgs;

typedef struct _CommandItem
{
	char *command;
	int uniqueCnt;
	PCOMMAND_FUNCTION function;
	PCommandArgs cmdArgs;
	char *comments;
} CommandItem, *PCommandItem;

//
// Global variables
//
CommandItem Commands[];
CommandItem HiddenCommands[];
char prompt[];

//
// Command processing functions
//
//void CommandLoop(void);
int CommandLoop(int argc, _TCHAR* argv[]);
int GetCommandItem(PCommandItem *ppCmdItem, int *pArgc, char* argv[]);
void PrintPrompt();
BOOL CompleteCommand(char *cmd, int *ndx);
void ParseString(char *str, BOOL useFirst, int end, char sep, int *pArgc, char *argv[], int maxArgs);
void ClearLine(int *pNdx);
void PrintHist(int *pNdx, int hist);

//
// Command functions
//
int CmdUnknownCommand(int argc, char* argv[]);
int CmdFindDevices(int argc, char* argv[]);
int CmdHelp(int argc, char *argv[]);
int CmdGetDefault(int argc, char* argv[]);
int CmdNumPorts(int argc, char* argv[]);
int CmdPrompt(int argc, char *argv[]);
int CmdIsReachable(int argc, char* argv[]);
int CmdIsReachableEx(int argc, char* argv[]);
int CmdQuery(int argc, char* argv[]);
int CmdQuit(int argc, char *argv[]);
int CmdGetVersions(int argc, char* argv[]);
int CmdPortDevice(int argc, char* argv[]);
int CmdPortInfo(int argc, char* argv[]);
int CmdPortMap(int argc, char* argv[]);
int CmdPortStatus(int argc, char* argv[]);
int CmdPortUsage(int argc, char* argv[]);
int CmdAddPort(int argc, char* argv[]);
int CmdRemovePort(int argc, char* argv[]);
int CmdSetDeviceNames(int argc, char* argv[]);
int CmdGetDeviceNames(int argc, char* argv[]);
int CmdSetPortInfo(int argc, char* argv[]);
int CmdSetDefPortInfo(int argc, char* argv[]);
int CmdAddService(int argc, char* argv[]);
int CmdRemoveService(int argc, char* argv[]);
int CmdSetService(int argc, char* argv[]);
int CmdImportCPC(int argc, char* argv[]);
int CmdExportCPC(int argc, char* argv[]);
int CmdRemoveAllPorts(int argc, char* argv[]);
int CmdLoopback(int argc, char* argv[]);
int CmdGetLogState(int argc, char* argv[]);
int CmdSetLogState(int argc, char* argv[]);

//
// Hidden Command functions
//
int CmdReleasePort(int argc, char* argv[]);

//
// Miscellaneous functions
//
void LogCprStatus(void);
void LogPortInfo(PortInfo* p);
void LogCprService(int port, CPR_SERVICE* p);
void LogCprServices(PortInfo *pPortInfo);
void PrintError(char *str, DWORD dwError);
DWORD GetErrorString(char *buf, DWORD bufSize, DWORD dwError);
void Trim(char *str, char tch);
void ParsePortInfo(int argc, char *argv[], int *a, int *r, int *c, int *d, int *u, int *b, int *t, int *L, int *k, int *m, int *i, int *p, int *s);

int LaunchThreads(
	int numThreads,
	BOOL random,
	BOOL printErrors,
	int dataSize,
	int numDataRuns,
	int comPorts[],
	int numComPorts,
	int printInterval,
	int numRuns,
	BOOL verbose
	);

#endif	// _CPRMGRCMD_H_