// Threads.cpp
//

#include "stdafx.h"
#include <strsafe.h>

#define BUF_SIZE			256
#define DATA_SIGNATURE		0x5F4E3D2C
#define FREE_SIGNATURE		0x01020304

#define PHASE_NONE			0
#define PHASE_OPENING		1
#define PHASE_WRITING		2
#define PHASE_READING		3
#define PHASE_CLOSING		4
#define PHASE_WAITING		5

typedef struct _ComPortData
{
	int comPort;
	ULONGLONG numOpens;
	ULONGLONG numWrites;
	ULONGLONG numReads;
	ULONGLONG numCloses;
	ULONGLONG numCompares;
	ULONGLONG numOpenErrs;
	ULONGLONG numWriteErrs;
	ULONGLONG numReadErrs;
	ULONGLONG numCompareErrs;
	ULONGLONG numCloseErrs;
} COMPORT_DATA, *PCOMPORT_DATA;

typedef struct _LoopbackData
{
	unsigned int signature;
	int threadNdx;
	int currentComPort;
	int phase;
	ULONGLONG numRuns;
	int numComPorts;
	PCOMPORT_DATA comPortData;
	int dataSize;
	int numDataRuns;
	ULONGLONG maxRuns;
	BOOL verbose;
} LOOPBACK_DATA, *PLOOPBACK_DATA;

typedef struct _LoopbackPrintData 
{
	unsigned int signature;
    int printInterval;
	BOOL random;
	int numThreads;
	int dataSize;
	int numDataRuns;
	int numComPorts;
	int *comPorts;
	PLOOPBACK_DATA *dataArray;
} LOOPBACK_PRINT_DATA, *PLOOPBACK_PRINT_DATA;

volatile BOOL quitPrint = FALSE;
volatile BOOL quit = FALSE;
volatile BOOL printInfo = FALSE;
volatile BOOL printHdr = FALSE;
volatile BOOL printDetails = FALSE;
volatile BOOL printErrors = TRUE;
volatile BOOL printingStats = TRUE;

void PrintThreadError(HANDLE hStdout, TCHAR *msgBuf, PLOOPBACK_DATA pData, char *str)
{
	if (printErrors)
	{
		char	str1[BUF_SIZE];
		DWORD   dwError = GetLastError();
		size_t	cchStringSize;
		DWORD	dwChars;

		if (GetErrorString(str1, BUF_SIZE, dwError))
		{
			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Thd %-2d Com %-3d: %s error (0x%x) %s\n"),
				pData->threadNdx, pData->currentComPort, str, dwError, str1);
		}
		else
		{
			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Thd %-2d Com %-3d: %s error (0x%x)\n"),
				pData->threadNdx, pData->currentComPort, str, dwError);
		}

		StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
		WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
	}
}

void PrintMsg(HANDLE hStdout, TCHAR *msgBuf, PLOOPBACK_DATA pData)
{
	if (pData->verbose)
	{
		size_t	cchStringSize;
		DWORD	dwChars;

		StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
		WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
	}
}

DWORD WINAPI LoopbackThreadProc( LPVOID lpParam )
{ 
	int i, j;
    HANDLE hStdout;
    PLOOPBACK_DATA pData;
    TCHAR msgBuf[BUF_SIZE];
    size_t cchStringSize;
    DWORD dwChars;
	char comStr[MAX_PORTS][12];
	HANDLE hComPort[MAX_PORTS];
	OVERLAPPED wol, rol, waitol;
	char Temp[20];
	DWORD DErr;
	DWORD numBytesWritten;
	DWORD numBytesRead;
	char *writeBuf;
	char *readBuf;
	int ch = 0x21;
	COMMTIMEOUTS cto = 
		{
			0,			// ReadIntervalTimeout
			0,			// ReadTotalTimeoutMultiplier
			10000,		// ReadTotalTimeoutConstant
			0,			// WriteTotalTimeoutMultiplier
			10000		// WriteTotalTimeoutConstant
		};

    // Cast the parameter to the correct data type.

    hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
    if( hStdout == INVALID_HANDLE_VALUE )
        return 1;

    pData = (PLOOPBACK_DATA)lpParam;

	if ((pData == NULL) || (pData->signature != DATA_SIGNATURE))
	{
		quit = TRUE;
		StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Invalid Data Buffer\n"));
		StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
		WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
		return 0;
	}

	PrintMsg(hStdout, TEXT("LoopbackThreadProc: Started.\n"), pData);

	writeBuf = (char*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(char) * pData->dataSize);

    if( writeBuf == NULL )
	{
		quit = TRUE;
		StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Unable to allocate memory for data buffers\n"));
		StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
		WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
		return 0;
	}

	readBuf = (char*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(char) * pData->dataSize);

    if( readBuf == NULL )
	{
		quit = TRUE;
	    HeapFree(GetProcessHeap(), 0, writeBuf);
		StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Unable to allocate memory for data buffers\n"));
		StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
		WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
		return 0;
	}

	for (i = 0; i < pData->dataSize; i++)
	{
		writeBuf[i] = ch++;
		if (ch > 0x7E)
			ch = 0x21;
	}

	for (i = 0; i < pData->numComPorts; i++)
	{
		StringCchPrintf(comStr[i], sizeof(comStr[i])/sizeof(TCHAR), TEXT("\\\\.\\COM%d"), pData->comPortData[i].comPort);
	}

	StringCchPrintf(Temp, sizeof(Temp), TEXT("Write%d"), pData->comPortData[i].comPort);
	memset(&wol,0,sizeof(OVERLAPPED));
	wol.hEvent = CreateEvent(NULL,FALSE,TRUE,Temp);

	StringCchPrintf(Temp, sizeof(Temp), TEXT("Read%d"), pData->comPortData[i].comPort);
	memset(&rol,0,sizeof(OVERLAPPED));
	rol.hEvent = CreateEvent(NULL,FALSE,FALSE,Temp);

	StringCchPrintf(Temp, sizeof(Temp), TEXT("Wait%d"), pData->comPortData[i].comPort);
	memset(&waitol,0,sizeof(OVERLAPPED));
	waitol.hEvent = CreateEvent(NULL,FALSE,FALSE,Temp);

	while (quit == false)
	{
		for (i = 0; (quit == FALSE) && (i < pData->numComPorts); i++)
		{
			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Open %s\n"), comStr[i]);
			PrintMsg(hStdout, msgBuf, pData);

			//
			// ========== Open ==========
			//
			pData->phase = PHASE_OPENING;
			pData->currentComPort = pData->comPortData[i].comPort;

			hComPort[i] = CreateFile(
								comStr[i],
								GENERIC_READ | GENERIC_WRITE,
								0,
								NULL,
								OPEN_EXISTING,
								FILE_FLAG_OVERLAPPED,
								NULL
								);

			if (hComPort[i] == INVALID_HANDLE_VALUE)
			{
				pData->comPortData[i].numOpenErrs++;
				PrintThreadError(hStdout, msgBuf, pData, "Open");
				continue;
			}

			pData->comPortData[i].numOpens++;

			SetCommTimeouts(hComPort[i], &cto);

			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Purge %s\n"), comStr[i]);
			PrintMsg(hStdout, msgBuf, pData);

			PurgeComm( hComPort[i], PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );

			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Write %s\n"), comStr[i]);
			PrintMsg(hStdout, msgBuf, pData);

			//
			// ========== Write ==========
			//
			ClearCommError(hComPort[i], &DErr, NULL);

			pData->phase = PHASE_WRITING;
			if (!WriteFile(hComPort[i], writeBuf, pData->dataSize, &numBytesWritten, &wol))
			{
				if (GetLastError() == ERROR_IO_PENDING)
				{
					if (WaitForSingleObject(wol.hEvent,1000) != WAIT_OBJECT_0)
					{
						pData->comPortData[i].numWriteErrs++;
						PrintThreadError(hStdout, msgBuf, pData, "Write(1)");
						numBytesWritten = 0;
					}
					else
					{
						GetOverlappedResult(hComPort[i], &wol, (LPDWORD)&numBytesWritten, FALSE);
					}
				}
				else
				{
					pData->comPortData[i].numWriteErrs++;
					PrintThreadError(hStdout, msgBuf, pData, "Write(2)");
					numBytesWritten = 0;
				}
			}

			ClearCommError(hComPort[i], &DErr, NULL);

			//
			// ========== Read ==========
			//
			if ((int)numBytesWritten == pData->dataSize)
			{
				int totDataRead = 0;

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Read %s\n"), comStr[i]);
				PrintMsg(hStdout, msgBuf, pData);

				pData->comPortData[i].numWrites++;
				memset(readBuf, 0, pData->dataSize);

				BOOL readErr = FALSE;
				pData->phase = PHASE_READING;
				do
				{
					if (quit)
						break;

					StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: ReadFile(1) %s, totDataRead %d, dataSize %d\n"),
						comStr[i], totDataRead, pData->dataSize);
					PrintMsg(hStdout, msgBuf, pData);

					ClearCommError(hComPort[i], &DErr, NULL);

					//
					// Wait for a character to be received.
					//
					do
					{
						DWORD readmask = 0;
						SetCommMask(hComPort[i], EV_RXCHAR);
						if (!WaitCommEvent(hComPort[i], &readmask, &waitol))
						{
							if (GetLastError() == ERROR_IO_PENDING)
							{
								PrintMsg(hStdout, TEXT("LoopbackThreadProc: PENDING Rx Wait\n"), pData);
								DWORD waitRetVal = WaitForSingleObject(waitol.hEvent, 1000);
							
								if (waitRetVal == WAIT_OBJECT_0)
								{
									StringCchPrintf(msgBuf, BUF_SIZE,
										TEXT("LoopbackThreadProc: WAIT_OBEJCT_0 Rx readMask %d\n"), readmask);
									PrintMsg(hStdout, msgBuf, pData);
									if (readmask & EV_RXCHAR)
									{
										GetOverlappedResult(hComPort[i], &rol, &numBytesRead, FALSE);
										break;
									}
								}
								else if (waitRetVal != WAIT_TIMEOUT)
								{
									pData->comPortData[i].numReadErrs++;
									PrintThreadError(hStdout, msgBuf, pData, "RxWait(1)");
									break;
								}
							}
						}
						else
						{
							StringCchPrintf(msgBuf, BUF_SIZE,
								TEXT("LoopbackThreadProc: DIRECT WAIT Rx readMask %d\n"), readmask);
							PrintMsg(hStdout, msgBuf, pData);
							if (readmask & EV_RXCHAR)
								break;
						}
						SetCommMask(hComPort[i], 0);
					} while (!quit);
					SetCommMask(hComPort[i], 0);

					if (!ReadFile(hComPort[i], readBuf + totDataRead, pData->dataSize - totDataRead, &numBytesRead, &rol))
					{
						if (GetLastError() == ERROR_IO_PENDING)
						{
							PrintMsg(hStdout, TEXT("LoopbackThreadProc: PENDING READ\n"), pData);
							DWORD waitRetVal = WaitForSingleObject(rol.hEvent, 1000);
							
							if (waitRetVal == WAIT_OBJECT_0)
							{
								PrintMsg(hStdout, TEXT("LoopbackThreadProc: WAIT_OBEJCT_0 Wait completed successfully\n"), pData);
								GetOverlappedResult(hComPort[i], &rol, &numBytesRead, FALSE);
							}
							else if (waitRetVal != WAIT_TIMEOUT)
							{
								pData->comPortData[i].numReadErrs++;
								PrintThreadError(hStdout, msgBuf, pData, "Read(1)");
								readErr = TRUE;
							}
						}
						else
						{
							pData->comPortData[i].numReadErrs++;
							PrintThreadError(hStdout, msgBuf, pData, "Read(2)");
							readErr = TRUE;
						}
					}
					else
					{
						PrintMsg(hStdout, TEXT("LoopbackThreadProc: DIRECT READ\n"), pData);
					}

					totDataRead += numBytesRead;

					StringCchPrintf(msgBuf, BUF_SIZE,
						TEXT("LoopbackThreadProc: ReadFile(2) %s, numRead %d, totDataRead %d, dataSize %d\n"),
						comStr[i], numBytesRead, totDataRead, pData->dataSize);
					PrintMsg(hStdout, msgBuf, pData);

					if (totDataRead == pData->dataSize)
					{
						pData->comPortData[i].numReads++;

						StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Compare %s\n"), comStr[i]);
						PrintMsg(hStdout, msgBuf, pData);

						//
						// ========== Compare ==========
						//
						for (j = 0; j < pData->dataSize; j++)
						{
							if (writeBuf[j] != readBuf[j])
							{
								break;
							}
						}
						if (j == pData->dataSize)
							pData->comPortData[i].numCompares++;
						else
							pData->comPortData[i].numCompareErrs++;
					}
				} while ((readErr == FALSE) && (totDataRead < pData->dataSize));

				ClearCommError(hComPort[i], &DErr, NULL);
			}

			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("LoopbackThreadProc: Close %s\n"), comStr[i]);
			PrintMsg(hStdout, msgBuf, pData);

			//
			// ========== Close ==========
			//
			pData->phase = PHASE_CLOSING;
			if (CloseHandle(hComPort[i]))
			{
				pData->comPortData[i].numCloses++;
			}
			else
			{
				pData->comPortData[i].numCloseErrs++;
				PrintThreadError(hStdout, msgBuf, pData, "Close");
			}

			pData->phase = PHASE_NONE;
		}

		pData->numRuns++;

		if ((pData->maxRuns > 0) && (pData->numRuns >= pData->maxRuns))
			break;
	}

	pData->phase = PHASE_WAITING;

	StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Thread %-2d waiting for statistics to be printed\n"), pData->threadNdx);
	StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
	WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

	while (printingStats)
	{
		Sleep(200);
	}

	CloseHandle(wol.hEvent);
	CloseHandle(rol.hEvent);

    HeapFree(GetProcessHeap(), 0, writeBuf);
    HeapFree(GetProcessHeap(), 0, readBuf);
	pData->signature = FREE_SIGNATURE;
    HeapFree(GetProcessHeap(), 0, pData->comPortData);
    HeapFree(GetProcessHeap(), 0, pData);

	PrintMsg(hStdout, TEXT("LoopbackThreadProc: Exiting\n"), pData);

	ExitThread(0);
    return 0; 
} 


DWORD WINAPI LoopbackThreadPrintProc( LPVOID lpParam )
{ 
	int i, j, w;
    HANDLE hStdout;
    PLOOPBACK_PRINT_DATA pPrData;
    PLOOPBACK_DATA pData;

    TCHAR msgBuf[BUF_SIZE];
    size_t cchStringSize;
    DWORD dwChars;

    hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
    if( hStdout == INVALID_HANDLE_VALUE )
        return 1;

    // Cast the parameter to the correct data type.

    pPrData = (PLOOPBACK_PRINT_DATA)lpParam;

    // Print the parameter values using thread-safe functions.

	printHdr = TRUE;
	printInfo = TRUE;

	while (quitPrint == FALSE)
	{
		for (w = 0; w < pPrData->printInterval * 2; w++)
		{
			int n = 0;
			for (i = 0; i < pPrData->numThreads; i++)
			{
				pData = pPrData->dataArray[i];
				if (pData->phase == PHASE_WAITING)
					n++;
			}
			if (n == pPrData->numThreads)
			{
				printf("\n\nStats will be printed here\n\n");
				quitPrint = TRUE;
				break;
			}

			if (quit)
			{
				break;
			}

			if (printInfo)
			{
				printInfo = FALSE;
				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\nRandom?            : %s\n"), (pPrData->random) ? "True" : "False");
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Data Size          : %d\n"), pPrData->dataSize);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Number of Data Runs: %d\n"), pPrData->numDataRuns);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Print interval     : %d\n"), pPrData->printInterval);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Number of Com Ports: %d"), pPrData->numComPorts);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				for (i = 0; i < pPrData->numComPorts; i++)
				{
					if ((i % 19) == 0)
					{
						StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n"), pPrData->numThreads);
						StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
						WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
					}

					StringCchPrintf(msgBuf, BUF_SIZE, TEXT("%4d"), pPrData->comPorts[i]);
					StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
					WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
				}

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\nNumber of Threads  : %d"), pPrData->numThreads);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				for (i = 0; i < pPrData->numThreads; i++)
				{
					pData = pPrData->dataArray[i];
					if (pData->signature == DATA_SIGNATURE)
					{
						int ph = pData->phase;
						StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n  Thread %3d is ACTIVE   : %3d ports : %s COM %-3d : # runs = %I64d"),
							i+1,
							pData->numComPorts,
							(ph == PHASE_OPENING) ? "Opening" :
							(ph == PHASE_WRITING) ? "Writing" :
							(ph == PHASE_READING) ? "Reading" :
							(ph == PHASE_CLOSING) ? "Closing" :
							(ph == PHASE_WAITING) ? "Waiting" : "None   ",
							pData->currentComPort,
							pData->numRuns);
						StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
						WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
					}
					else
					{
						StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\nThread %3d is INACTIVE :"), i+1);
						StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
						WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
					}
				}

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n\nPress 'I' to print basic information\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Press 'H' to print header\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Press 'E' to 'toggle' print error mode. Currently %s\n"), (printErrors) ? "On" : "Off");
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Press 'D' to print detailed information\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Press 'Q' to quit test\n\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				printHdr = TRUE;
			}

			if (printDetails)
			{
				printDetails = FALSE;
				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\nThr Com    Opens   Writes    Reads   Closes OpErrs WrErrs RdErrs ClErrs CmErrs"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n------------------------------------------------------------------------------\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
			
				for (i = 0; i < pPrData->numThreads; i++)
				{
					pData = pPrData->dataArray[i];
					if (pData->signature == DATA_SIGNATURE)
					{
						for (j = 0; j < pData->numComPorts; j++)
						{
							PCOMPORT_DATA pComPortData = &pData->comPortData[j];
							StringCchPrintf(
								msgBuf, 
								BUF_SIZE, 
								TEXT("%3d %3d%9I64d%9I64d%9I64d%9I64d%7I64d%7I64d%7I64d%7I64d%7I64d\n"),
								i+1,
								pComPortData->comPort,
								pComPortData->numOpens,
								pComPortData->numWrites,
								pComPortData->numReads,
								pComPortData->numCloses,
								pComPortData->numOpenErrs,
								pComPortData->numWriteErrs,
								pComPortData->numReadErrs,
								pComPortData->numCloseErrs,
								pComPortData->numCompareErrs);
							
							StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
							WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
						}

						printHdr = TRUE;
					}
				}
			}

			if (printHdr)
			{
				printHdr = FALSE;
				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n              Opens         Writes          Reads         Closes       Compares"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

				StringCchPrintf(msgBuf, BUF_SIZE, TEXT("\n-------------------------------------------------------------------------------\n"));
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
			}

			Sleep(500);
		}

		if (quit)
			printInfo = TRUE;

		if (pPrData->signature == DATA_SIGNATURE)
		{
			ULONGLONG totOpens = 0;
			ULONGLONG totWrites = 0;
			ULONGLONG totReads = 0;
			ULONGLONG totCloses = 0;
			ULONGLONG totCompares = 0;
			ULONGLONG totOpenErrs = 0;
			ULONGLONG totWriteErrs = 0;
			ULONGLONG totReadErrs = 0;
			ULONGLONG totCompareErrs = 0;
			ULONGLONG totCloseErrs = 0;

			for (i = 0; i < pPrData->numThreads; i++)
			{
				pData = pPrData->dataArray[i];
				if (pData->signature == DATA_SIGNATURE)
				{
					for (j = 0; j < pData->numComPorts; j++)
					{
						PCOMPORT_DATA pComPortData = &pData->comPortData[j];
						totOpens += pComPortData->numOpens;
						totWrites += pComPortData->numWrites;
						totReads += pComPortData->numReads;
						totCloses += pComPortData->numCloses;
						totCompares += pComPortData->numCompares;
						totOpenErrs += pComPortData->numOpenErrs;
						totWriteErrs += pComPortData->numWriteErrs;
						totReadErrs += pComPortData->numReadErrs;
						totCloseErrs += pComPortData->numCloseErrs;
						totCompareErrs += pComPortData->numCompareErrs;
					}
				}
			}

			StringCchPrintf(
				msgBuf, 
				BUF_SIZE, 
				TEXT("Pass%15I64d%15I64d%15I64d%15I64d%15I64d\n"),
				totOpens,
				totWrites,
				totReads,
				totCloses,
				totCompares);
			StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
			WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);

			if (totOpenErrs | totWriteErrs | totReadErrs | totCloseErrs | totCompareErrs)
			{
				StringCchPrintf(
					msgBuf, 
					BUF_SIZE, 
					TEXT("Fail%15I64d%15I64d%15I64d%15I64d%15I64d\n"),
					totOpenErrs,
					totWriteErrs,
					totReadErrs,
					totCloseErrs,
					totCompareErrs);
				StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
				WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
			}
		}
		else
		{
			StringCchPrintf(msgBuf, BUF_SIZE, TEXT("Invalid data in print thread\n"));
			StringCchLength(msgBuf, BUF_SIZE, &cchStringSize);
			WriteConsole(hStdout, msgBuf, (DWORD)cchStringSize, &dwChars, NULL);
			break;
		}
	}

    // Free the memory allocated by the caller for the thread 
    // data structure.

	printingStats = FALSE;
	pPrData->signature = FREE_SIGNATURE;
    HeapFree(GetProcessHeap(), 0, pPrData->dataArray);
    HeapFree(GetProcessHeap(), 0, pPrData);

	ExitThread(0);
    return 0; 
} 

int LaunchThreads(
	int numThreads,
	BOOL random,
	BOOL locPrintErrors,
	int dataSize,
	int numDataRuns,
	int comPorts[],
	int numComPorts,
	int printInterval,
	int numRuns,
	BOOL verbose
	)
{
	int	i, j;
    PLOOPBACK_DATA pData;
	PLOOPBACK_PRINT_DATA pPrData;
    DWORD *dwThreadId;
    HANDLE *hThread; 
    DWORD dwPrintThreadId;
    HANDLE hPrintThread; 
	int numPerThread = numComPorts / numThreads;
	int remain = numComPorts % numThreads;
	int count;
	int numComPortsSent = 0;

	quit = FALSE;
	quitPrint = FALSE;
	printErrors = locPrintErrors;
	printingStats = TRUE;

	if (verbose)
		printf("LaunchThreads: Started.\n");

	// Allocate thread id memory, make room for data and print threads
	dwThreadId = (DWORD*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
        sizeof(DWORD) * (numThreads + 1));

    if( dwThreadId == NULL )
	{
		printf("Unable to allocate memory for threads\n");
		return 0;
	}

	hThread = (HANDLE*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
        sizeof(HANDLE) * (numThreads + 1));

    if( dwThreadId == NULL )
	{
	    HeapFree(GetProcessHeap(), 0, dwThreadId);
		printf("Unable to allocate memory for threads\n");
		return 0;
	}

	pPrData = (PLOOPBACK_PRINT_DATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
					sizeof(LOOPBACK_PRINT_DATA));

    if( pPrData == NULL )
	{
	    HeapFree(GetProcessHeap(), 0, dwThreadId);
	    HeapFree(GetProcessHeap(), 0, hThread);
		printf("Unable to allocate memory for threads\n");
		return 0;
	}

	pPrData->signature = DATA_SIGNATURE;
	pPrData->numThreads = numThreads;
	pPrData->printInterval = printInterval;
	pPrData->dataSize = dataSize;
	pPrData->numComPorts = numComPorts;
	pPrData->comPorts = comPorts;
	pPrData->numDataRuns = numDataRuns;
	pPrData->random = random;
	pPrData->dataArray = (PLOOPBACK_DATA*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
        sizeof(PLOOPBACK_DATA) * numThreads);

    if( pPrData->dataArray == NULL )
	{
	    HeapFree(GetProcessHeap(), 0, pPrData);
	    HeapFree(GetProcessHeap(), 0, dwThreadId);
	    HeapFree(GetProcessHeap(), 0, hThread);
		printf("Unable to allocate memory for threads\n");
		return 0;
	}

    // Create worker threads.
    for ( i = 0; i < numThreads; i++ )
    {
        // Allocate memory for thread data.

        pData = (PLOOPBACK_DATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
                sizeof(LOOPBACK_DATA));

        if( pData == NULL )
		{
			quit = TRUE;
		    HeapFree(GetProcessHeap(), 0, pPrData->dataArray);
		    HeapFree(GetProcessHeap(), 0, pPrData);
		    HeapFree(GetProcessHeap(), 0, dwThreadId);
		    HeapFree(GetProcessHeap(), 0, hThread);
			printf("Unable to allocate memory for threads\n");
			return 0;
		}

		pData->signature = DATA_SIGNATURE;
		pPrData->dataArray[i] = pData;

        // Assign com ports to threads
		count = numPerThread + ((remain) ? 1 : 0);
		if (remain) remain--;

		pData->comPortData = (PCOMPORT_DATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
			sizeof(COMPORT_DATA) * count);

        if( pData->comPortData == NULL )
		{
			quit = TRUE;
		    HeapFree(GetProcessHeap(), 0, pData);
		    HeapFree(GetProcessHeap(), 0, pPrData->dataArray);
		    HeapFree(GetProcessHeap(), 0, pPrData);
		    HeapFree(GetProcessHeap(), 0, dwThreadId);
		    HeapFree(GetProcessHeap(), 0, hThread);
			printf("Unable to allocate memory for threads\n");
			return 0;
		}

		if (random == FALSE)
		{
			for (j = 0; j < count; j++)
			{
				pData->comPortData[j].comPort = comPorts[numComPortsSent + j];
			}
		}
		else
		{
			for (j = 0; j < count; j++)
			{
				pData->comPortData[j].comPort = comPorts[numComPortsSent + j];
			}
		}

		numComPortsSent += count;
		pData->numComPorts = count;
        pData->threadNdx = i + 1;
		pData->dataSize = dataSize;
		pData->numDataRuns = numDataRuns;
		pData->maxRuns = numRuns;
		pData->verbose = verbose;

		if (verbose)
			printf("LaunchThreads: CreateThread %d.\n", i+1);

		// Launch data thread

        hThread[i] = CreateThread( 
            NULL,                // default security attributes
            0,                   // use default stack size  
            LoopbackThreadProc,  // thread function 
            pData,               // argument to thread function 
            0,                   // use default creation flags 
            &dwThreadId[i]);     // returns the thread identifier 
 
        // Check the return value for success. 
 
        if (hThread[i] == NULL) 
        {
			quit = TRUE;
		    HeapFree(GetProcessHeap(), 0, pPrData->dataArray);
		    HeapFree(GetProcessHeap(), 0, pPrData);
		    HeapFree(GetProcessHeap(), 0, pData->comPortData);
		    HeapFree(GetProcessHeap(), 0, pData);
		    HeapFree(GetProcessHeap(), 0, dwThreadId);
		    HeapFree(GetProcessHeap(), 0, hThread);
			printf("Unable to Spawn threads\n");
			return 0;
        }
    }

	if (verbose)
		printf("LaunchThreads: Launch print thread.\n");

	// Launch print thread

	hPrintThread = CreateThread( 
		NULL,                // default security attributes
		0,                   // use default stack size  
		LoopbackThreadPrintProc,  // thread function 
		pPrData,               // argument to thread function 
		0,                   // use default creation flags 
		&dwPrintThreadId);     // returns the thread identifier 

	// Check the return value for success. 

	if (hPrintThread == NULL) 
	{
		quit = TRUE;
		HeapFree(GetProcessHeap(), 0, pPrData->dataArray);
		HeapFree(GetProcessHeap(), 0, pPrData);
		printf("Unable to Spawn print thread\n");
	}
	else
	{
		if (verbose)
			printf("LaunchThreads: Waiting for threads to terminate.\n");

		// Wait until all threads have terminated.

		while (TRUE)
		{
			if (_kbhit())
			{
				int ch = _getch();
				if (toupper(ch) == 'Q')
				{
					quit = TRUE;
					printf("Signalled data threads to terminate!\n");
				}

				if (toupper(ch) == 'I')
				{
					printInfo = TRUE;
				}

				if (toupper(ch) == 'H')
				{
					printHdr = TRUE;
				}

				if (toupper(ch) == 'D')
				{
					printDetails = TRUE;
				}

				if (toupper(ch) == 'E')
				{
					printErrors = !printErrors;
					printf("Error printing has been turned %s\n", (printErrors) ? "On" : "Off");
				}
			}

			if (quit && (quitPrint == FALSE))
			{
				if (WaitForMultipleObjects(numThreads, hThread, TRUE, 100) != WAIT_TIMEOUT)
				{
					quitPrint = TRUE;
					printf("All data threads have terminated!\nSignalled print thread to terminate!\n");
				}
			}

			if (quitPrint)
			{
				if (WaitForSingleObject(hPrintThread, 100) != WAIT_TIMEOUT)
				{
					printf("Print thread has terminated!\n");
					break;
				}
			}

			Sleep(500);
		}
	}

	if (verbose)
		printf("LaunchThreads: Closing all thread handles.\n");

    // Close all thread handles upon completion.

    for(i=0; i<=numThreads; i++)
    {
        CloseHandle(hThread[i]);
    }

	HeapFree(GetProcessHeap(), 0, dwThreadId);
	HeapFree(GetProcessHeap(), 0, hThread);

	printf("Exiting loopback test!\n");

	return 0;
}

////////////////////////////////////////////////////////////
//
// CPR API Functions that are not called in this program... yet?:
//
// CloseComPort
// ReleasePort
//
////////////////////////////////////////////////////////////

