// CprMonCmd.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

DWORD WINAPI EventLogThreadProc(
  LPVOID lpParameter
  );

BOOL quitThread = FALSE;
BOOL threadComplete = FALSE;

char unknown[50];
char tmp[200];
char time[100];
LARGE_INTEGER perfFreq;
LARGE_INTEGER startTicks;

BOOL verbose = FALSE;
BOOL hex = FALSE;

char *ULongToIP(ULONG ipAddr, char *ipStr);
char *EventLogString(ULONG eventType, ULONG subType);
char *StatusString(ULONG status);
char *Msg(char *msg, ULONG type, ULONG len);
char *TimeString(LARGE_INTEGER ticks);
BOOL ParseFilterFile(TCHAR *filterFile, 	CPR_FILTER_DATA *pCprFilter);

char *usage = "Usage: cprmoncmd [-v -h -f<filter file> -p -?]\n"
			  "       -v is verbose mode\n"
			  "       -h display data in hexidecimal format (defaults to ASCII)\n"
			  "       -f<filter file> use specified filter file,\n"
			  "         otherwise all events will be trapped.\n"
			  "       -p prints current filter conditions\n"
			  "       -? prints this usage\n"
			  "       try piping the output to a file like this:\n"
			  "         cprmoncmd -v -h > cprmon.log\n";

int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE	statusThread;
	DWORD	threadId;
	int		ch;
	int		i;
	CPR_FILTER_DATA cprFilter;
	BOOL	printFilter = FALSE;
	TCHAR	*filterFile = NULL;

	for (i = 0; i < argc; i++)
	{
		if (!wcscmp(argv[i], L"-v"))
			verbose = TRUE;

		if (!wcscmp(argv[i], L"-h"))
			hex = TRUE;

		if (!wcscmp(argv[i], L"-p"))
			printFilter = TRUE;

		if (!wcsncmp(argv[i], L"-f", 2))
			filterFile = &argv[i][2];

		if (!wcscmp(argv[i], L"-?"))
		{
			printf(usage);
			return -1;
		}
	}

	if ((verbose == FALSE) && (hex == FALSE) && (filterFile == NULL) && (printFilter == FALSE) && (argc > 1))
	{
		printf(usage);
		return -2;
	}

	if (ParseFilterFile(filterFile, &cprFilter) == FALSE)
		return -3;

	if (printFilter)
	{
		printf("CprComPorts[%d] =\n", NUM_COM_ULONGS);
		for (i = 0; i < NUM_COM_ULONGS; i++)
		{
			printf("%08X\n", cprFilter.CprComPorts[i]);
		}
		printf("CprEventLogFilter = %08X\n", cprFilter.CprEventLogFilter);
		printf("CprELIoctlFilter  = %08X\n", cprFilter.CprELIoctlFilter);
		printf("CprEventFlags     = %08X\n", cprFilter.CprEventFlags);
	}

	perfFreq.QuadPart = 0;
	QueryPerformanceFrequency(&perfFreq);
	CprEL_GetCurrentPerformanceCounter(&startTicks);

	CprEL_StartEventRead();

	CprEL_SetFilterData(&cprFilter);

	statusThread = CreateThread(NULL, 0, EventLogThreadProc, NULL, 0, &threadId);

	printf("\nListening for Events. Press Enter to quit\n\n");

	ch = getchar();

	quitThread = TRUE;
	CprEL_CancelEventRead();

	while (threadComplete == FALSE)
	{
		Sleep(100);
	}

	return 0;
}

//
// Event Thread
//
DWORD WINAPI EventLogThreadProc(
	LPVOID lpParameter
	)
{
	CPR_EVENT_LOG eventLog;
	char ipStr[20];
	DWORD line = 1;

	memset(&eventLog, 0, sizeof(CPR_EVENT_LOG));

	if (verbose)
	{
		printf("Line #      COM  Time            IP Address       Port   Event                Status                     Message\n");
		printf("----------  ---  --------------  ---------------  -----  -------------------- -------------------------- ----------------\n");
	}
	else
	{
		printf("COM Time           IP Address      Port  Event      Status     Message\n");
		printf("--- -------------- --------------- ----- ---------- ---------- ----------------\n");
	}

	do
	{
		int status = CprEL_WaitOnEventRead(&eventLog);

		if (eventLog.flags & CPR_EVENT_FLAG_CANCELLED)
		{
			printf("CprEL_WaitOnEventRead returned CPR_EVENT_FLAG_CANCELLED\n");
			break;
		}
		else
		{
			if (verbose)
			{
				printf("%010d  %-3d  %s  %-15s  %-5d  %-20s %-26s %s\n", 
					line++,
					eventLog.comPort,
					TimeString(eventLog.ticks),
					ULongToIP(eventLog.ipAddr, ipStr),
					eventLog.tcpPort,
					EventLogString(eventLog.type, eventLog.subType),
					StatusString(eventLog.status),
					Msg(eventLog.msg, eventLog.type, 300));	// Only print 14 bytes of msg
			}
			else
			{
				printf("%-3d %s %-15s %-5d %-10s %-10s %s\n", 
					eventLog.comPort,
					TimeString(eventLog.ticks),
					ULongToIP(eventLog.ipAddr, ipStr),
					eventLog.tcpPort,
					EventLogString(eventLog.type, eventLog.subType),
					StatusString(eventLog.status),
					Msg(eventLog.msg, eventLog.type, 16));	// Only print 14 bytes of msg
			}
		}
	} while (quitThread == FALSE);

	threadComplete = TRUE;

	return 0;
}


BOOL ParseFilterFile(TCHAR *filterFile, CPR_FILTER_DATA *pCprFilter)
{
	int		i;
	BOOL	bTmp;
	BOOL	retVal = FALSE;

	// Clear pCprFilter
	pCprFilter->CprELIoctlFilter = 0;
	pCprFilter->CprEventFlags = 0;
	pCprFilter->CprEventLogFilter = 0;
	pCprFilter->CprTraceFlags = 0;

	for (i = 0; i < NUM_COM_ULONGS; i++)
	{
		pCprFilter->CprComPorts[i] = 0;
	}

	if (filterFile == NULL)
	{
		//
		// Programs written to use the Event Log functionality
		// would provide some way to allow the user to
		// specify which events he would like to listen to.
		// This function listens for every event.
		//
		pCprFilter->CprEventLogFilter =
			CPR_EVENT_TYPE_NONE						|
			CPR_EVENT_TYPE_OPEN						|
			CPR_EVENT_TYPE_CLOSE					|
			CPR_EVENT_TYPE_READ						|
			CPR_EVENT_TYPE_WRITE					|
			CPR_EVENT_TYPE_CONNECT					|
			CPR_EVENT_TYPE_DISCONNECT				|
			CPR_EVENT_TYPE_RECONNECTING				|
			CPR_EVENT_TYPE_RECONNECTED				|
			CPR_EVENT_TYPE_WRITE_TIMEOUT			|
			CPR_EVENT_TYPE_XON						|
			CPR_EVENT_TYPE_XOFF						|
			CPR_EVENT_TYPE_RFC2217					|
			CPR_EVENT_TYPE_WAIT_RXCHAR				|
			CPR_EVENT_TYPE_WAIT_RXFLAG				|
			CPR_EVENT_TYPE_WAIT_TXEMPTY				|
			CPR_EVENT_TYPE_WAIT_CTS					|
			CPR_EVENT_TYPE_WAIT_DSR					|
			CPR_EVENT_TYPE_WAIT_RLSD				|
			CPR_EVENT_TYPE_WAIT_BREAK				|
			CPR_EVENT_TYPE_WAIT_ERR					|
			CPR_EVENT_TYPE_WAIT_RING				|
			CPR_EVENT_TYPE_WAIT_RX80FULL			|
			CPR_EVENT_TYPE_DATA_RECEIVE				|
			CPR_EVENT_TYPE_DATA_TRANSMIT			|
			CPR_EVENT_TYPE_READ_REQUEST				|
			CPR_EVENT_TYPE_READ_TIMEOUT				|
			CPR_EVENT_TYPE_PENDED_READ_RELEASED		|
			CPR_EVENT_TYPE_DEBUG					|
			CPR_EVENT_TYPE_READ_OVERFLOW;

		pCprFilter->CprELIoctlFilter = 
			CPR_ET_IOCTL_PURGE						|
			CPR_ET_IOCTL_GET_HANDFLOW				|
			CPR_ET_IOCTL_SET_HANDFLOW				|
			CPR_ET_IOCTL_GET_MODEM_STATUS			|
			CPR_ET_IOCTL_GET_DTRRTS					|
			CPR_ET_IOCTL_GET_COMM_STATUS			|
			CPR_ET_IOCTL_GET_PROPERTIES				|
			CPR_ET_IOCTL_SET_BAUDRATE				|
			CPR_ET_IOCTL_GET_BAUDRATE				|
			CPR_ET_IOCTL_SET_MODEM_CONTROL			|
			CPR_ET_IOCTL_GET_MODEM_CONTROL			|
			CPR_ET_IOCTL_SET_LINE_CONTROL			|
			CPR_ET_IOCTL_XOFF_COUNTER				|
			CPR_ET_IOCTL_GET_STATS					|
			CPR_ET_IOCTL_SET_TIMEOUTS				|
			CPR_ET_IOCTL_GET_TIMEOUTS				|
			CPR_ET_IOCTL_SET_CHARS					|
			CPR_ET_IOCTL_GET_CHARS					|
			CPR_ET_IOCTL_SET_DTR					|
			CPR_ET_IOCTL_CLEAR_DTR					|
			CPR_ET_IOCTL_SET_RTS					|
			CPR_ET_IOCTL_CLEAR_RTS					|
			CPR_ET_IOCTL_SET_XON					|
			CPR_ET_IOCTL_SET_XOFF					|
			CPR_ET_IOCTL_SET_BREAK_ON				|
			CPR_ET_IOCTL_SET_BREAK_OFF				|
			CPR_ET_IOCTL_SET_QUEUE_SIZE				|
			CPR_ET_IOCTL_GET_WAIT_MASK				|
			CPR_ET_IOCTL_SET_WAIT_MASK				|
			CPR_ET_IOCTL_WAIT_ON_MASK				|
			CPR_ET_IOCTL_IMMEDIATE_CHAR				|
			CPR_ET_IOCTL_GET_LINE_CONTROL;

		pCprFilter->CprEventFlags = 
			CPR_EVENT_FLAG_PASS_UP					| 
			CPR_EVENT_FLAG_SEND_READ_DATA			| 
			CPR_EVENT_FLAG_SEND_WRITE_DATA;

		pCprFilter->CprTraceFlags = 0;	// Not used by Cpr Driver

		//	// =================================================
		//	// Just examples of how to use the CprComPort Macros:
		//	//	CPR_SET_PORT, CPR_IS_PORT_SET, and CPR_CLR_PORT
		//	//
		//	for (i = 0; i < NUM_COM_ULONGS; i++)
		//	{
		//		cprFilter.CprComPorts[i] = 0x0;
		//	}
		//
		//	CPR_SET_PORT(cprFilter.CprComPorts, 5);
		//	printf("cpa %08X\n", cprFilter.CprComPorts[0]);
		//
		//	printf("cpa %d\n", CPR_IS_PORT_SET(cprFilter.CprComPorts, 5));
		//
		//	CPR_CLR_PORT(cprFilter.CprComPorts, 5);
		//	printf("cpa %X\n", cprFilter.CprComPorts[0]);
		//	// End of examples.
		//	// =================================================

		// This for loop will set all the com ports 1 - 256
		for (i = 0; i < NUM_COM_ULONGS; i++)
		{
			pCprFilter->CprComPorts[i] = 0xFFFFFFFF;
		}

		retVal = TRUE;
	}
	else
	{
		// Open and parse filterFile
		HANDLE hFile;
		DWORD lowFileSize;
		DWORD highFileSize;
		BOOL bRc;
		char *fileBuf = NULL;
		char *line,
             *next_line;
		DWORD dwNumRead;
		int state;
#define STATE_NONE			0
#define STATE_COMPORTS		1
#define STATE_EVENTS		2
#define STATE_EVENT_IOCTLS	3
#define STATE_EVENT_FLAGS	4

		hFile = CreateFile(
			filterFile,
			GENERIC_READ,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL, // sec attr
			OPEN_EXISTING,
			0,             // no attributes
			NULL);         // no template

		if (hFile == (HANDLE)-1)
		{
			printf("FAIL: Open of file '%ws' failed rc: %d\n", filterFile, GetLastError());
			return FALSE;
		}

		lowFileSize = GetFileSize(hFile, &highFileSize);

		if (lowFileSize == INVALID_FILE_SIZE)
		{
			printf("FAIL: Getting file size. failed rc: %d\n", GetLastError());
			CloseHandle(hFile);
			return FALSE;
		}

		if (lowFileSize > 0x10000)
		{
			printf("FAIL: Invalid Filter File\n");
			CloseHandle(hFile);
			return FALSE;
		}

		// Allocate memory for file and read it into memory:
		fileBuf = (char*)malloc(lowFileSize);
		if (fileBuf == NULL)
		{
			printf("FAIL: allocating memory for file rc: %d\n", GetLastError());
			CloseHandle(hFile);
			return FALSE;
		}

		bRc = ReadFile(
			hFile,
			fileBuf,
			lowFileSize,
			&dwNumRead,
			NULL);

		if (!bRc)
		{
			printf("\nFAIL: cannot Read From file '%s':%d\n", filterFile, GetLastError());
			CloseHandle(hFile);
			return FALSE;
		}

		state = STATE_NONE;

		line = strtok_s(fileBuf, "\n\r", &next_line);
		while (line != NULL)
		{
			size_t len;

			// remove spaces and tabs from front and back
			for ( ; *line == ' ' || *line == '\t'; line++)
				;

			len = strlen(line);
			if (len)
			{
				for (len--; len; len--)
				{
					if ((line[len] != ' ') && (line[len] != '\t'))
						break;
					//printf("----%c----\n", line[len]);
					line[len] = '\0';
				}
			}

			if (!strncmp(line, "COMPORTS:", 9))
			{
				state = STATE_COMPORTS;
				//printf("Found COMPORTS:\n");
			}
			else if (!strncmp(line, "EVENTS:", 7))
			{
				state = STATE_EVENTS;
				//printf("Found EVENTS:\n");
			}
			else if (!strncmp(line, "EVENT_IOCTLS:", 13))
			{
				state = STATE_EVENT_IOCTLS;
				//printf("Found EVENT_IOCTLS:\n");
			}
			else if (!strncmp(line, "EVENT_FLAGS:", 12))
			{
				state = STATE_EVENT_FLAGS;
				//printf("Found EVENT_FLAGS:\n");
			}
			else
			{
				if (strncmp(line, "//", 2))
				{
					switch (state)
					{
						case STATE_COMPORTS:
							if (!strcmp(line, "ALL"))
							{
								for (i = 0; i < NUM_COM_ULONGS; i++)
								{
									pCprFilter->CprComPorts[i] = 0xFFFFFFFF;
								}
							}
							else
							{
								int port = atoi(line);
								if ((port > 0) && (port <= 256))
								{
									CPR_SET_PORT(pCprFilter->CprComPorts, port);
								}
							}

							//printf("STATE_COMPORTS: %s\n", line);
							break;

						case STATE_EVENTS:
							if (!strcmp(line, "ALL"))
								pCprFilter->CprEventLogFilter = 0xFFFFFFFF;
							else if (!strcmp(line, "CPR_EVENT_TYPE_OPEN"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_OPEN;
							else if (!strcmp(line, "CPR_EVENT_TYPE_CLOSE"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_CLOSE;
							else if (!strcmp(line, "CPR_EVENT_TYPE_READ"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_READ;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WRITE"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WRITE;
							else if (!strcmp(line, "CPR_EVENT_TYPE_CONNECT"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_CONNECT;
							else if (!strcmp(line, "CPR_EVENT_TYPE_DISCONNECT"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_DISCONNECT;
							else if (!strcmp(line, "CPR_EVENT_TYPE_RECONNECTING"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_RECONNECTING;
							else if (!strcmp(line, "CPR_EVENT_TYPE_RECONNECTED"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_RECONNECTED;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WRITE_TIMEOUT"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WRITE_TIMEOUT;
							else if (!strcmp(line, "CPR_EVENT_TYPE_XON"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_XON;
							else if (!strcmp(line, "CPR_EVENT_TYPE_XOFF"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_XOFF;
							else if (!strcmp(line, "CPR_EVENT_TYPE_RFC2217"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_RFC2217;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_RXCHAR"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_RXCHAR;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_RXFLAG"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_RXFLAG;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_TXEMPTY"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_TXEMPTY;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_CTS"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_CTS;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_DSR"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_DSR;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_RLSD"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_RLSD;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_BREAK"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_BREAK;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_ERR"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_ERR;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_RING"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_RING;
							else if (!strcmp(line, "CPR_EVENT_TYPE_WAIT_RX80FULL"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_WAIT_RX80FULL;
							else if (!strcmp(line, "CPR_EVENT_TYPE_DATA_RECEIVE"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_DATA_RECEIVE;
							else if (!strcmp(line, "CPR_EVENT_TYPE_DATA_TRANSMIT"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_DATA_TRANSMIT;
							else if (!strcmp(line, "CPR_EVENT_TYPE_READ_REQUEST"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_READ_REQUEST;
							else if (!strcmp(line, "CPR_EVENT_TYPE_READ_TIMEOUT"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_READ_TIMEOUT;
							else if (!strcmp(line, "CPR_EVENT_TYPE_PENDED_READ_RELEASED"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_PENDED_READ_RELEASED;
							else if (!strcmp(line, "CPR_EVENT_TYPE_DEBUG"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_DEBUG;
							else if (!strcmp(line, "CPR_EVENT_TYPE_READ_OVERFLOW"))
								pCprFilter->CprEventLogFilter |= CPR_EVENT_TYPE_READ_OVERFLOW;
							//printf("STATE_EVENTS: %s\n", line);
							break;

						case STATE_EVENT_IOCTLS:
							if (!strcmp(line, "ALL"))
								pCprFilter->CprELIoctlFilter = 0xFFFFFFFF;
							else if (!strcmp(line, "CPR_ET_IOCTL_PURGE"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_PURGE;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_HANDFLOW"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_HANDFLOW;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_HANDFLOW"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_HANDFLOW;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_MODEM_STATUS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_MODEM_STATUS;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_DTRRTS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_DTRRTS;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_COMM_STATUS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_COMM_STATUS;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_PROPERTIES"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_PROPERTIES;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_BAUDRATE"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_BAUDRATE;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_BAUDRATE"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_BAUDRATE;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_MODEM_CONTROL"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_MODEM_CONTROL;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_MODEM_CONTROL"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_MODEM_CONTROL;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_LINE_CONTROL"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_LINE_CONTROL;
							else if (!strcmp(line, "CPR_ET_IOCTL_XOFF_COUNTER"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_XOFF_COUNTER;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_STATS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_STATS;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_TIMEOUTS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_TIMEOUTS;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_TIMEOUTS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_TIMEOUTS;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_CHARS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_CHARS;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_CHARS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_CHARS;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_DTR"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_DTR;
							else if (!strcmp(line, "CPR_ET_IOCTL_CLEAR_DTR"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_CLEAR_DTR;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_RTS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_RTS;
							else if (!strcmp(line, "CPR_ET_IOCTL_CLEAR_RTS"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_CLEAR_RTS;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_XON"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_XON;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_XOFF"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_XOFF;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_BREAK_ON"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_BREAK_ON;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_BREAK_OFF"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_BREAK_OFF;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_QUEUE_SIZE"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_QUEUE_SIZE;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_WAIT_MASK"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_WAIT_MASK;
							else if (!strcmp(line, "CPR_ET_IOCTL_SET_WAIT_MASK"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_SET_WAIT_MASK;
							else if (!strcmp(line, "CPR_ET_IOCTL_WAIT_ON_MASK"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_WAIT_ON_MASK;
							else if (!strcmp(line, "CPR_ET_IOCTL_IMMEDIATE_CHAR"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_IMMEDIATE_CHAR;
							else if (!strcmp(line, "CPR_ET_IOCTL_GET_LINE_CONTROL"))
								pCprFilter->CprELIoctlFilter |= CPR_ET_IOCTL_GET_LINE_CONTROL;
							//printf("STATE_EVENT_IOCTLS: %s\n", line);
							break;

						case STATE_EVENT_FLAGS:
							if (!strcmp(line, "ALL"))
								pCprFilter->CprEventFlags = 0xFFFFFFFF;
							else if (!strcmp(line, "CPR_EVENT_FLAG_PASS_UP"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_PASS_UP;
							else if (!strcmp(line, "CPR_EVENT_FLAG_SEND_READ_DATA"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_SEND_READ_DATA;
							else if (!strcmp(line, "CPR_EVENT_FLAG_SEND_WRITE_DATA"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_SEND_WRITE_DATA;
							else if (!strcmp(line, "CPR_EVENT_FLAG_DBGPRINT"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_DBGPRINT;
							else if (!strcmp(line, "CPR_EVENT_FLAG_FAIL"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_FAIL;
							else if (!strcmp(line, "CPR_EVENT_FLAG_ABORT"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_ABORT;
							else if (!strcmp(line, "CPR_EVENT_FLAG_CANCELLED"))
								pCprFilter->CprEventFlags |= CPR_EVENT_FLAG_CANCELLED;
							//printf("STATE_EVENT_FLAGS: %s\n", line);
							break;
					}
				}
			}

			line = strtok_s(NULL, "\n\r", &next_line);
		}

		CloseHandle(hFile);
		retVal = TRUE;
	}

	bTmp = FALSE;
	for (i = 0; i < NUM_COM_ULONGS; i++)
	{
		if (pCprFilter->CprComPorts[i])
		{
			bTmp = TRUE;
			break;
		}
	}

	if (bTmp == FALSE)
	{
		printf("\nMust specifiy at least 1 Com Port to filter on\n");
		printf("Specify 'ALL' in the COMPORTS: section to listen to all Com Ports\n");
		retVal = FALSE;
	}

	if ((pCprFilter->CprELIoctlFilter == 0) && (pCprFilter->CprEventLogFilter == 0))
	{
		printf("\nMust specify at least 1 Event Filter\n");
		retVal = FALSE;
	}

	if (pCprFilter->CprEventFlags == 0)
	{
		printf("\nMust specify at least 1 Event Flag\n");
		retVal = FALSE;
	}

	return retVal;
}

char *ULongToIP(ULONG ipAddr, char *ipStr)
{
	sprintf_s(ipStr, 20, "%d.%d.%d.%d",
		ipAddr         & 0xFF,
		(ipAddr >> 8)  & 0xFF,
		(ipAddr >> 16) & 0xFF,
		(ipAddr >> 24) & 0xFF);

	return ipStr;
}

char *EventLogString(ULONG type, ULONG subType)
{
	ULONG unknownType = 0;

	switch(type)
	{
	case CPR_EVENT_TYPE_OPEN:					return "Open";
	case CPR_EVENT_TYPE_CLOSE:					return "Close";
	case CPR_EVENT_TYPE_READ:					return "Read";
	case CPR_EVENT_TYPE_WRITE:					return "Write";
	case CPR_EVENT_TYPE_CONNECT:				return "Connect";
	case CPR_EVENT_TYPE_DISCONNECT:				return "Disconnect";
	case CPR_EVENT_TYPE_RECONNECTING:			return (verbose) ? "Reconnecting" : "Reconnecti";
	case CPR_EVENT_TYPE_RECONNECTED:			return (verbose) ? "Reconnected" : "Reconnecte";
	case CPR_EVENT_TYPE_WRITE_TIMEOUT:			return (verbose) ? "Write Timeout" : "Write Time";
	case CPR_EVENT_TYPE_XON:					return "Xon";
	case CPR_EVENT_TYPE_XOFF:					return "Xoff";
	case CPR_EVENT_TYPE_RFC2217:				return "RFC2217";
	case CPR_EVENT_TYPE_WAIT_RXCHAR:			return (verbose) ? "Wait RxChar" : "Wait RxCha";
	case CPR_EVENT_TYPE_WAIT_RXFLAG:			return (verbose) ? "Wait RxFlag" : "Wait RxFla";
	case CPR_EVENT_TYPE_WAIT_TXEMPTY:			return (verbose) ? "Wait TxEmpty" : "Wait TxEmp";
	case CPR_EVENT_TYPE_WAIT_CTS:				return "Wait CTS";
	case CPR_EVENT_TYPE_WAIT_DSR:				return "Wait DSR";
	case CPR_EVENT_TYPE_WAIT_RLSD:				return "Wait RLSD";
	case CPR_EVENT_TYPE_WAIT_BREAK:				return "Wait Break";
	case CPR_EVENT_TYPE_WAIT_ERR:				return "Wait Error";
	case CPR_EVENT_TYPE_WAIT_RING:				return "Wait Ring";
	case CPR_EVENT_TYPE_WAIT_RX80FULL:			return (verbose) ? "Wait Rx80Full" : "Wait Rx80F";
	case CPR_EVENT_TYPE_DATA_RECEIVE:			return (verbose) ? "Data Received" : "Data Recei";
	case CPR_EVENT_TYPE_DATA_TRANSMIT:			return (verbose) ? "Data Transmit" : "Data Trans";
	case CPR_EVENT_TYPE_READ_REQUEST:			return (verbose) ? "Read Request" : "Read Reque";
	case CPR_EVENT_TYPE_READ_TIMEOUT:			return (verbose) ? "Read Timeoout" : "Read Timeo";
	case CPR_EVENT_TYPE_PENDED_READ_RELEASED:	return (verbose) ? "Pended Read Released" : "Pend Read ";
	case CPR_EVENT_TYPE_DEBUG:					return "Debug";
	case CPR_EVENT_TYPE_NONE:
		switch(subType)
		{
		case CPR_ET_IOCTL_PURGE:				return "Purge";
		case CPR_ET_IOCTL_GET_HANDFLOW:			return (verbose) ? "Get Handflow" : "Get Handfl";
		case CPR_ET_IOCTL_SET_HANDFLOW:			return (verbose) ? "Set Handflow" : "Set Handfl";
		case CPR_ET_IOCTL_GET_MODEM_STATUS:		return (verbose) ? "Get Modem Status" : "Get Modem ";
		case CPR_ET_IOCTL_GET_DTRRTS:			return (verbose) ? "Get DTR RTS" : "Get DTR RT";
		case CPR_ET_IOCTL_GET_COMM_STATUS:		return (verbose) ? "Get Com Status" : "Get Com St";
		case CPR_ET_IOCTL_GET_PROPERTIES:		return (verbose) ? "Get Properties" : "Get Proper";
		case CPR_ET_IOCTL_SET_BAUDRATE:			return (verbose) ? "Set Baudrate" : "Set Baudra";
		case CPR_ET_IOCTL_GET_BAUDRATE:			return (verbose) ? "Get Baudrate" : "Get Baudra";
		case CPR_ET_IOCTL_SET_MODEM_CONTROL:	return (verbose) ? "Set Modem Control" : "Set Modem ";
		case CPR_ET_IOCTL_GET_MODEM_CONTROL:	return (verbose) ? "Get Modem Control" : "Get Modem ";
		case CPR_ET_IOCTL_SET_LINE_CONTROL:		return (verbose) ? "Set Line Control" : "Set Line C";
		case CPR_ET_IOCTL_XOFF_COUNTER:			return (verbose) ? "Xoff Counter" : "Xoff Count";
		case CPR_ET_IOCTL_GET_STATS:			return "Get Stats";
		case CPR_ET_IOCTL_SET_TIMEOUTS:			return (verbose) ? "Set Timeouts" : "Set Timeou";
		case CPR_ET_IOCTL_GET_TIMEOUTS:			return (verbose) ? "Get Timeouts" : "Get Timeou";
		case CPR_ET_IOCTL_SET_CHARS:			return "Set Chars";
		case CPR_ET_IOCTL_GET_CHARS:			return "Get Chars";
		case CPR_ET_IOCTL_SET_DTR:				return "Set DTR";
		case CPR_ET_IOCTL_CLEAR_DTR:			return "Clear DTR";
		case CPR_ET_IOCTL_SET_RTS:				return "Set RTS";
		case CPR_ET_IOCTL_CLEAR_RTS:			return "Clear RTS";
		case CPR_ET_IOCTL_SET_XON:				return "Set Xon";
		case CPR_ET_IOCTL_SET_XOFF:				return "Set Xoff";
		case CPR_ET_IOCTL_SET_BREAK_ON:			return (verbose) ? "Set Break On " : "Set Br On ";
		case CPR_ET_IOCTL_SET_BREAK_OFF:		return (verbose) ? "Set Break Off" : "Set Br Off";
		case CPR_ET_IOCTL_SET_QUEUE_SIZE:		return (verbose) ? "Set Queue Size" : "Set QueSiz";
		case CPR_ET_IOCTL_GET_WAIT_MASK:		return (verbose) ? "Get Wait Mask" : "Get Wait M";
		case CPR_ET_IOCTL_SET_WAIT_MASK:		return (verbose) ? "Set Wait Mask" : "Set Wait M";
		case CPR_ET_IOCTL_WAIT_ON_MASK:			return (verbose) ? "Wait On Mask" : "Wait On Ma";
		case CPR_ET_IOCTL_IMMEDIATE_CHAR:		return (verbose) ? "Immediate Char" : "Immediate ";
		case CPR_ET_IOCTL_GET_LINE_CONTROL:		return (verbose) ? "Get Line Control" : "Get Line C";
		default:
			unknownType = subType;
			break;
		}
		break;
	default:
		unknownType = type;
		break;
	}

	sprintf_s(unknown, 50, "Unknown:%08X", unknownType);
	return unknown;
}

char *StatusString(ULONG status)
{
	switch (status)
	{
    case 0x00000000:	return "Success";
    case 0x00000102:	return "Timeout";
    case 0x00000103:	return "Pending";
    case 0xC0000001:	return "Unsuccessful";
    case 0xC0000005:	return "Access Violation";
    case 0xC0000008:	return "Invalid Handle";
    case 0xC000000C:	return "Timer Not Cancelled";
    case 0xC000000D:	return "Invalid Parameter";
    case 0xC000000E:	return "No Such Device";
    case 0xC000000F:	return "No Such File";
    case 0xC0000010:	return (verbose) ? "Invalid Device Request" : "Invalid Device Req.";
    case 0xC0000016:	return (verbose) ? "More Processing Required" : "More Processing Req.";
    case 0xC0000017:	return "No Memory";
    case 0xC0000021:	return "Already Committed";
    case 0xC0000022:	return "Access Denied";
    case 0xC0000023:	return "Buffer Too Small";
    case 0xC0000041:	return (verbose) ? "Port Connection Refused" : "Port Connection Ref.";
    case 0xC000009A:	return (verbose) ? "Insufficient Resources" : "Insufficient Res.";
    case 0xC00000B5:	return "I/O Timeout";
    case 0xC00000E5:	return "Internal Error";
    case 0xC0000120:	return "Cancelled";
    case 0xC0000184:	return "Invalid Device State";
    case 0xC000020A:	return (verbose) ? "Address Already Exists" : "Addr Already Exists";
    case 0xC0000235:	return "Handle Not Closable";
    case 0xC0000236:	return "Connection Refused";
    case 0xC0000237:	return "Graceful Disconnect";
    case 0xC0000238:	return (verbose) ? "Address Already Associated" : "Addr Already Assoc.";
    case 0xC0000239:	return (verbose) ? "Address Not Associated" : "Addr Not Associated";
    case 0xC000023A:	return "Connection Invalid";
    case 0xC000023B:	return "Connection Active";
    case 0xC000023C:	return "Network Unreachable";
    case 0xC000023D:	return "Host Unreachable";
    case 0xC000023E:	return "Protocol Unreachable";
    case 0xC000023F:	return "Port Unreachable";
    case 0xC0000240:	return "Request Aborted";
    case 0xC0000241:	return "Connection Aborted";
	}

	sprintf_s(unknown, 50, "Unknown:%08X", status);
	return unknown;
}

char msgTmp[512];

char *Msg(char *msg, ULONG type, ULONG len)
{
	ULONG n;
	char *str = tmp;
	char *verb = "read";
	size_t ndx = 0;

	if (msg == NULL)
		return "";

	if ((type == CPR_EVENT_TYPE_READ) ||
		(type == CPR_EVENT_TYPE_WRITE) ||
		(type == CPR_EVENT_TYPE_DATA_RECEIVE) ||
		(type == CPR_EVENT_TYPE_DATA_TRANSMIT) ||
		(type == CPR_EVENT_TYPE_PENDED_READ_RELEASED))
	{
		// nbytes is number of bytes in msg
		ULONG nbytes  = ((ULONG)msg[1] << 8) + (ULONG)msg[0];
		// bufSize is the number of bytes in the read, write, etc.
		// buffer read from or written to the device server.
		// In otherwords, msg may not contain all the data.
		ULONG bufSize = ((ULONG)msg[3] << 8) + (ULONG)msg[2];

		if (nbytes > len)
			nbytes = len;

		if (verbose)
		{
            if (type == CPR_EVENT_TYPE_WRITE)
                verb = "written";
            else if (type == CPR_EVENT_TYPE_DATA_RECEIVE)
                verb = "received";
            else if (type == CPR_EVENT_TYPE_DATA_TRANSMIT)
                verb = "transmitted";
            else if (type == CPR_EVENT_TYPE_PENDED_READ_RELEASED)
                verb = "released";

			sprintf_s(msgTmp, 512, "Displaying %d of %d bytes %s ", nbytes, bufSize, verb);
			ndx = strlen(msgTmp);
		}

		for (n = 0; n < nbytes; n++)
		{
			if (hex)
			{
				sprintf_s(&msgTmp[ndx], 512-ndx, "%02X ", (unsigned char)msg[n+4]);
				ndx += 3;
			}
			else if (!isprint(msg[n+4]))
			{
				msgTmp[ndx++] = '.';
			}
			else
			{
				msgTmp[ndx++] = msg[n+4];
			}
		}

		msgTmp[ndx] = '\0';
		return msgTmp;
	}

	if (strlen(msg) > len)
		msg[len] = '\0';
	return msg;
}

char *TimeString(LARGE_INTEGER ticks)
{
    __int64 val;
	__int64 ticks64 = ticks.QuadPart;
	__int64 startTicks64 = startTicks.QuadPart;
	__int64 perfFreq64 = perfFreq.QuadPart;
    int hours;
    int minutes;
    int seconds;
    int milliseconds;

    if (ticks64 > startTicks64)
        val = ticks64 - startTicks64;
    else
        val = startTicks64 - ticks64;

    val = val * 100000;
    val = val / perfFreq64;

    milliseconds = (int)(val % 100000);
    val = val / 100000;

    seconds = (int)(val % 60);
    val = val / 60;

    minutes = (int)(val % 60);
    val = val / 60;

    hours = (int)val;

	sprintf_s(time, 100, "%02d:%02d:%02d.%05d",
            hours,
            minutes,
            seconds,
            milliseconds);

	return time;
}
