// CprStatusCmd.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

DWORD WINAPI StatusThreadProc(
  LPVOID lpParameter
  );

BOOL quitThread = FALSE;
BOOL threadComplete = FALSE;

char *ComStatusString(PPortStatus pPortStatus);
char *NetworkStatusString(PPortStatus pPortStatus, char *buf, int bufSize);
char *Usage = "Usage: CprStatusCmd <All | <Com List>>\n"
			  "         Com List Example: CprStatus 4 5 6 23\n"
			  "                           Status only on ports 4, 5, 6 and 23\n";

ULONG	comList[NUM_COM_ULONGS];


int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE	statusThread;
	DWORD	threadId;
	int		ch;
	int		i;

	if (argc == 1)
	{
		printf(Usage);
		return -1;
	}

	if ((*argv[1] == 'a') || (*argv[1] == 'A'))
	{
		for (i = 0; i < NUM_COM_ULONGS; i++)
		{
			comList[i] = 0xFFFFFFFF;
		}
	}
	else
	{
		for (i = 1; i < argc; i++)
		{
			int com = _wtoi(argv[i]);
			int portMinusOne = com - 1;
			unsigned int comNdx = portMinusOne >> 5;
			unsigned int comMask = (1 << (portMinusOne % 32));
			// Set bit to one
			comList[comNdx] |= comMask;
		}
	}

	statusThread = CreateThread(NULL, 0, StatusThreadProc, NULL, 0, &threadId);

	printf("\nListening for status changes. Press Enter to quit\n\n");

	ch = getchar();

	quitThread = TRUE;
	CprCancelComStatus();

	while (threadComplete == FALSE)
	{
		Sleep(100);
	}

	return 0;
}


#define MAX_NET_BUF	200

DWORD WINAPI StatusThreadProc(
	LPVOID lpParameter
	)
{
	ULONG	comStatus[NUM_COM_ULONGS];
	int		comStatusLen = NUM_COM_ULONGS * sizeof(ULONG);
	int		i, j;
	ULONG	mask;
	ULONG	change;
	char	buf[MAX_NET_BUF];

	memset(comStatus, 0, comStatusLen);

	do
	{
		CprGetComStatus(comStatus, &comStatusLen, TRUE);

		// Look for changes in the port status
		//
		for (i = 0; (i < NUM_COM_ULONGS) && (quitThread == FALSE); i++)
		{
			// Look at each ULONG
			if (comStatus[i] != 0)
			{
				// We have a change in status
				// Look at each BIT

				mask   = 0x00000001;
				change = comStatus[i];

				for (j = 0; (j < 32) && (quitThread == FALSE); j++)
				{
					if (change & mask & comList[i])
					{
						// There is a change...
						// Need to get the status from the driver

						PortStatus portStatus;
						portStatus.portNumber = (i * 32) + j + 1;
						if (CprGetPortStatus(&portStatus) == CPR_OK)
						{
							printf("Com %3d: %-8s Network: %s\n",
								portStatus.portNumber,
								ComStatusString(&portStatus),
								NetworkStatusString(&portStatus, buf, MAX_NET_BUF));
						}
					}
					mask <<= 1;
				}
			}
		}
	} while (quitThread == FALSE);

	threadComplete = TRUE;

	return 0;
}


//#define CPR_COM_STATUS_CLOSED				0
//#define CPR_COM_STATUS_OPENING			1
//#define CPR_COM_STATUS_OPEN				2
//#define CPR_COM_STATUS_CLOSING			3
//#define CPR_COM_STATUS_CLEANUP			4
//#define CPR_COM_STATUS_ERROR				5

char *ComStatusString(PPortStatus pPortStatus)
{
	switch (pPortStatus->status)
	{
		case CPR_COM_STATUS_CLOSED:		return "Closed";
		case CPR_COM_STATUS_OPENING:	return "Opening";
		case CPR_COM_STATUS_OPEN:		return "Open";
		case CPR_COM_STATUS_CLOSING:	return "Closing";
		case CPR_COM_STATUS_CLEANUP:	return "Cleanup";
		case CPR_COM_STATUS_ERROR:		return "Error";
	}

	return "Invalid";
}

//#define CPR_NETWORK_STATUS_DISCONNECTED	0
//#define CPR_NETWORK_STATUS_CONNECTING		1
//#define CPR_NETWORK_STATUS_RECONNECTING	2
//#define CPR_NETWORK_STATUS_CONNECTED		3
//#define CPR_NETWORK_STATUS_LISTENING		4			// Listening in Listen Mode
//#define CPR_NETWORK_STATUS_NEGOTIATING	5			// RFC2217 Negotiations
//#define CPR_NETWORK_STATUS_VERIFYING		6			// Verifying if a Lantronix Device
//#define CPR_NETWORK_STATUS_ERROR			7

char *NetworkStatusString(PPortStatus pPortStatus, char *buf, int bufSize)
{
	char tmpBuf[MAX_NET_BUF];
	int  numChars = 0;

	if (bufSize > 0)
		buf[0] = '\0';
	else
		return "Invalid Buffer in NetworkStatusString";

	switch (pPortStatus->networkStatus)
	{
		case CPR_NETWORK_STATUS_DISCONNECTED:
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Disconnected");
			break;
		case CPR_NETWORK_STATUS_CONNECTING:		
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Connecting to %ws:%d", pPortStatus->Address, pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_RECONNECTING:	
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Reconnecting to %ws:%d", pPortStatus->Address, pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_CONNECTED:		
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Connected to %ws:%d", pPortStatus->Address, pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_LISTENING:		
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Listening on TCP Port %d", pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_NEGOTIATING:	
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Negotiating RFC2217 with %ws:%d", pPortStatus->Address, pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_VERIFYING:		
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Verifying %ws:%d is a valid device", pPortStatus->Address, pPortStatus->Port);
			break;
		case CPR_NETWORK_STATUS_ERROR:			
			numChars = sprintf_s(tmpBuf, MAX_NET_BUF, "Error with %ws:%d", pPortStatus->Address, pPortStatus->Port);
			break;
	}

	if ((numChars + 1) > bufSize)
		numChars = bufSize - 1;

	strncpy_s(buf, MAX_NET_BUF, tmpBuf, numChars + 1);
	buf[bufSize-1] = '\0';	// just in case.

	return buf;
}
